# autogenerated by genrb from WorldState.msg. Do not edit.
require 'ros/message'

require "geometry_msgs/Wrench"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "geometry_msgs/Twist"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Vector3"
require "geometry_msgs/Pose"

module Gazebo_msgs

class WorldState <::ROS::Message
  def self.md5sum
    "de1a9de3ab7ba97ac0e9ec01a4eb481e"
  end

  def self.type
    "gazebo_msgs/WorldState"
  end

  def has_header?
    true
  end

  def message_definition
    "# This is a message that holds data necessary to reconstruct a snapshot of the world
#
# = Approach to Message Passing =
# The state of the world is defined by either
#   1. Inertial Model pose, twist
#      * kinematic data - connectivity graph from Model to each Link
#      * joint angles
#      * joint velocities
#      * Applied forces - Body wrench
#        * relative transform from Body to each collision Geom
# Or
#   2. Inertial (absolute) Body pose, twist, wrench
#      * relative transform from Body to each collision Geom - constant, so not sent over wire
#      * back compute from canonical body info to get Model pose and twist.
#
# Chooing (2.) because it matches most physics engines out there
#   and is simpler.
#
# = Future =
# Consider impacts on using reduced coordinates / graph (parent/child links) approach
#   constraint and physics solvers.
#
# = Application =
# This message is used to do the following:
#   * reconstruct the world and objects for sensor generation
#   * stop / start simulation - need pose, twist, wrench of each body
#   * collision detection - need pose of each collision geometry.  velocity/acceleration if 
#
# = Assumptions =
# Assuming that each (physics) processor node locally already has
#   * collision information - Trimesh for Geoms, etc
#   * relative transforms from Body to Geom - this is assumed to be fixed, do not send oved wire
#   * inertial information - does not vary in time
#   * visual information - does not vary in time
#

Header header

string[] name
geometry_msgs/Pose[] pose
geometry_msgs/Twist[] twist
geometry_msgs/Wrench[] wrench

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: geometry_msgs/Twist
# This expresses velocity in free space broken into its linear and angular parts.
Vector3  linear
Vector3  angular

================================================================================
MSG: geometry_msgs/Vector3
# This represents a vector in free space. 

float64 x
float64 y
float64 z
================================================================================
MSG: geometry_msgs/Wrench
# This represents force in free space, separated into
# its linear and angular parts.
Vector3  force
Vector3  torque

"
  end
  attr_accessor :header, :name, :pose, :twist, :wrench

  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['std_msgs/Header','string[]','geometry_msgs/Pose[]','geometry_msgs/Twist[]','geometry_msgs/Wrench[]']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [std_msgs/Header] :header initialize value
  # @option args [string[]] :name initialize value
  # @option args [geometry_msgs/Pose[]] :pose initialize value
  # @option args [geometry_msgs/Twist[]] :twist initialize value
  # @option args [geometry_msgs/Wrench[]] :wrench initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:header]
      @header = args[:header]
    else
      @header = Std_msgs::Header.new
    end
    if args[:name]
      @name = args[:name]
    else
      @name = []
    end
    if args[:pose]
      @pose = args[:pose]
    else
      @pose = []
    end
    if args[:twist]
      @twist = args[:twist]
    else
      @twist = []
    end
    if args[:wrench]
      @wrench = args[:wrench]
    else
      @wrench = []
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_L3.pack(@header.seq, @header.stamp.secs, @header.stamp.nsecs))
      _x = @header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @name.length
      buff.write(@@struct_L.pack(length))
      for val1 in @name
        length = val1.length
        buff.write([length, val1].pack("La#{length}"))
      end
      length = @pose.length
      buff.write(@@struct_L.pack(length))
      for val1 in @pose
        _v3755 = val1.position
        _x = _v3755
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v3756 = val1.orientation
        _x = _v3756
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @twist.length
      buff.write(@@struct_L.pack(length))
      for val1 in @twist
        _v3757 = val1.linear
        _x = _v3757
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v3758 = val1.angular
        _x = _v3758
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
      length = @wrench.length
      buff.write(@@struct_L.pack(length))
      for val1 in @wrench
        _v3759 = val1.force
        _x = _v3759
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v3760 = val1.torque
        _x = _v3760
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @header == nil
        @header = Std_msgs::Header.new
      end
      if @pose == nil
        @pose = Geometry_msgs::Pose[].new
      end
      if @twist == nil
        @twist = Geometry_msgs::Twist[].new
      end
      if @wrench == nil
        @wrench = Geometry_msgs::Wrench[].new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@header.seq, @header.stamp.secs, @header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @name = []
      length.times do
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1 = str[start..(end_point-1)]
        @name.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @pose = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v3761 = val1.position
        _x = _v3761
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v3762 = val1.orientation
        _x = _v3762
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @pose.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @twist = []
      length.times do
        val1 = Geometry_msgs::Twist.new
        _v3763 = val1.linear
        _x = _v3763
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v3764 = val1.angular
        _x = _v3764
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @twist.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @wrench = []
      length.times do
        val1 = Geometry_msgs::Wrench.new
        _v3765 = val1.force
        _x = _v3765
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v3766 = val1.torque
        _x = _v3766
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @wrench.push(val1)
      end
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
