# autogenerated by genrb from DatabaseModelPose.msg. Do not edit.
require 'ros/message'

require "geometry_msgs/PoseStamped"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Pose"

module Household_objects_database_msgs

class DatabaseModelPose <::ROS::Message
  def self.md5sum
    "6bc39ef48ca57cc7d4341cfc13a5a110"
  end

  def self.type
    "household_objects_database_msgs/DatabaseModelPose"
  end

  def has_header?
    false
  end

  def message_definition
    "# Informs that a specific model from the Model Database has been 
# identified at a certain location

# the database id of the model
int32 model_id

# if the object was recognized by the ORK pipeline, its type will be in here
# if this is not empty, then the string in here will be converted to a household_objects_database id
# leave this empty if providing an id in the model_id field
object_recognition_msgs/ObjectType type

# the pose that it can be found in
geometry_msgs/PoseStamped pose

# a measure of the confidence level in this detection result
float32 confidence

# the name of the object detector that generated this detection result
string detector_name

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: geometry_msgs/PoseStamped
# A Pose with reference coordinate frame and timestamp
Header header
Pose pose

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

"
  end
  attr_accessor :model_id, :type, :pose, :confidence, :detector_name

  @@struct_l = ::ROS::Struct.new("l")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d7f = ::ROS::Struct.new("d7f")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['int32','object_recognition_msgs/ObjectType','geometry_msgs/PoseStamped','float32','string']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [int32] :model_id initialize value
  # @option args [object_recognition_msgs/ObjectType] :type initialize value
  # @option args [geometry_msgs/PoseStamped] :pose initialize value
  # @option args [float32] :confidence initialize value
  # @option args [string] :detector_name initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:model_id]
      @model_id = args[:model_id]
    else
      @model_id = 0
    end
    if args[:type]
      @type = args[:type]
    else
      @type = Object_recognition_msgs::ObjectType.new
    end
    if args[:pose]
      @pose = args[:pose]
    else
      @pose = Geometry_msgs::PoseStamped.new
    end
    if args[:confidence]
      @confidence = args[:confidence]
    else
      @confidence = 0.0
    end
    if args[:detector_name]
      @detector_name = args[:detector_name]
    else
      @detector_name = ''
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_l.pack(@model_id))
      _x = @type.key
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @type.db
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L3.pack(@pose.header.seq, @pose.header.stamp.secs, @pose.header.stamp.nsecs))
      _x = @pose.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d7f.pack(@pose.pose.position.x, @pose.pose.position.y, @pose.pose.position.z, @pose.pose.orientation.x, @pose.pose.orientation.y, @pose.pose.orientation.z, @pose.pose.orientation.w, @confidence))
      _x = @detector_name
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @type == nil
        @type = Object_recognition_msgs::ObjectType.new
      end
      if @pose == nil
        @pose = Geometry_msgs::PoseStamped.new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('l')
      (@model_id,) = @@struct_l.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.key = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.db = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@pose.header.seq, @pose.header.stamp.secs, @pose.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @pose.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d7f')
      (@pose.pose.position.x, @pose.pose.position.y, @pose.pose.position.z, @pose.pose.orientation.x, @pose.pose.orientation.y, @pose.pose.orientation.z, @pose.pose.orientation.w, @confidence,) = @@struct_d7f.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @detector_name = str[start..(end_point-1)]
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
