# autogenerated by genrb from GraspableObjectList.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "sensor_msgs/PointField"
require "geometry_msgs/Point32"
require "geometry_msgs/PoseStamped"
require "sensor_msgs/PointCloud2"
require "sensor_msgs/RegionOfInterest"
require "sensor_msgs/PointCloud"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "manipulation_msgs/GraspableObject"
require "geometry_msgs/Point"
require "manipulation_msgs/SceneRegion"
require "shape_msgs/Mesh"
require "sensor_msgs/CameraInfo"
require "household_objects_database_msgs/DatabaseModelPose"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Vector3"
require "sensor_msgs/ChannelFloat32"
require "geometry_msgs/Pose"
require "sensor_msgs/Image"

module Manipulation_msgs

class GraspableObjectList <::ROS::Message
  def self.md5sum
    "d67571f2982f1b7115de1e0027319109"
  end

  def self.type
    "manipulation_msgs/GraspableObjectList"
  end

  def has_header?
    false
  end

  def message_definition
    "manipulation_msgs/GraspableObject[] graspable_objects

#Information required for visualization

sensor_msgs/Image image
sensor_msgs/CameraInfo camera_info

#Holds a single mesh for each recognized graspable object, an empty mesh otherwise
shape_msgs/Mesh[] meshes

#pose to transform the frame of the clusters/object poses into camera coordinates
geometry_msgs/Pose reference_to_camera

================================================================================
MSG: manipulation_msgs/GraspableObject
# an object that the object_manipulator can work on

# a graspable object can be represented in multiple ways. This message
# can contain all of them. Which one is actually used is up to the receiver
# of this message. When adding new representations, one must be careful that
# they have reasonable lightweight defaults indicating that that particular
# representation is not available.

# the tf frame to be used as a reference frame when combining information from
# the different representations below
string reference_frame_id

# potential recognition results from a database of models
# all poses are relative to the object reference pose
household_objects_database_msgs/DatabaseModelPose[] potential_models

# the point cloud itself
sensor_msgs/PointCloud cluster

# a region of a PointCloud2 of interest
SceneRegion region

# the name that this object has in the collision environment
string collision_name
================================================================================
MSG: household_objects_database_msgs/DatabaseModelPose
# Informs that a specific model from the Model Database has been 
# identified at a certain location

# the database id of the model
int32 model_id

# if the object was recognized by the ORK pipeline, its type will be in here
# if this is not empty, then the string in here will be converted to a household_objects_database id
# leave this empty if providing an id in the model_id field
object_recognition_msgs/ObjectType type

# the pose that it can be found in
geometry_msgs/PoseStamped pose

# a measure of the confidence level in this detection result
float32 confidence

# the name of the object detector that generated this detection result
string detector_name

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: geometry_msgs/PoseStamped
# A Pose with reference coordinate frame and timestamp
Header header
Pose pose

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: sensor_msgs/PointCloud
# This message holds a collection of 3d points, plus optional additional
# information about each point.

# Time of sensor data acquisition, coordinate frame ID.
Header header

# Array of 3d points. Each Point32 should be interpreted as a 3d point
# in the frame given in the header.
geometry_msgs/Point32[] points

# Each channel should have the same number of elements as points array,
# and the data in each channel should correspond 1:1 with each point.
# Channel names in common practice are listed in ChannelFloat32.msg.
ChannelFloat32[] channels

================================================================================
MSG: geometry_msgs/Point32
# This contains the position of a point in free space(with 32 bits of precision).
# It is recommeded to use Point wherever possible instead of Point32.  
# 
# This recommendation is to promote interoperability.  
#
# This message is designed to take up less space when sending
# lots of points at once, as in the case of a PointCloud.  

float32 x
float32 y
float32 z
================================================================================
MSG: sensor_msgs/ChannelFloat32
# This message is used by the PointCloud message to hold optional data
# associated with each point in the cloud. The length of the values
# array should be the same as the length of the points array in the
# PointCloud, and each value should be associated with the corresponding
# point.

# Channel names in existing practice include:
#   \"u\", \"v\" - row and column (respectively) in the left stereo image.
#              This is opposite to usual conventions but remains for
#              historical reasons. The newer PointCloud2 message has no
#              such problem.
#   \"rgb\" - For point clouds produced by color stereo cameras. uint8
#           (R,G,B) values packed into the least significant 24 bits,
#           in order.
#   \"intensity\" - laser or pixel intensity.
#   \"distance\"

# The channel name should give semantics of the channel (e.g.
# \"intensity\" instead of \"value\").
string name

# The values array should be 1-1 with the elements of the associated
# PointCloud.
float32[] values

================================================================================
MSG: manipulation_msgs/SceneRegion
# Point cloud
sensor_msgs/PointCloud2 cloud

# Indices for the region of interest
int32[] mask

# One of the corresponding 2D images, if applicable
sensor_msgs/Image image

# The disparity image, if applicable
sensor_msgs/Image disparity_image

# Camera info for the camera that took the image
sensor_msgs/CameraInfo cam_info

# a 3D region of interest for grasp planning
geometry_msgs/PoseStamped  roi_box_pose
geometry_msgs/Vector3 	   roi_box_dims

================================================================================
MSG: sensor_msgs/PointCloud2
# This message holds a collection of N-dimensional points, which may
# contain additional information such as normals, intensity, etc. The
# point data is stored as a binary blob, its layout described by the
# contents of the \"fields\" array.

# The point cloud data may be organized 2d (image-like) or 1d
# (unordered). Point clouds organized as 2d images may be produced by
# camera depth sensors such as stereo or time-of-flight.

# Time of sensor data acquisition, and the coordinate frame ID (for 3d
# points).
Header header

# 2D structure of the point cloud. If the cloud is unordered, height is
# 1 and width is the length of the point cloud.
uint32 height
uint32 width

# Describes the channels and their layout in the binary data blob.
PointField[] fields

bool    is_bigendian # Is this data bigendian?
uint32  point_step   # Length of a point in bytes
uint32  row_step     # Length of a row in bytes
uint8[] data         # Actual point data, size is (row_step*height)

bool is_dense        # True if there are no invalid points

================================================================================
MSG: sensor_msgs/PointField
# This message holds the description of one point entry in the
# PointCloud2 message format.
uint8 INT8    = 1
uint8 UINT8   = 2
uint8 INT16   = 3
uint8 UINT16  = 4
uint8 INT32   = 5
uint8 UINT32  = 6
uint8 FLOAT32 = 7
uint8 FLOAT64 = 8

string name      # Name of field
uint32 offset    # Offset from start of point struct
uint8  datatype  # Datatype enumeration, see above
uint32 count     # How many elements in the field

================================================================================
MSG: sensor_msgs/Image
# This message contains an uncompressed image
# (0, 0) is at top-left corner of image
#

Header header        # Header timestamp should be acquisition time of image
                     # Header frame_id should be optical frame of camera
                     # origin of frame should be optical center of cameara
                     # +x should point to the right in the image
                     # +y should point down in the image
                     # +z should point into to plane of the image
                     # If the frame_id here and the frame_id of the CameraInfo
                     # message associated with the image conflict
                     # the behavior is undefined

uint32 height         # image height, that is, number of rows
uint32 width          # image width, that is, number of columns

# The legal values for encoding are in file src/image_encodings.cpp
# If you want to standardize a new string format, join
# ros-users@lists.sourceforge.net and send an email proposing a new encoding.

string encoding       # Encoding of pixels -- channel meaning, ordering, size
                      # taken from the list of strings in include/sensor_msgs/image_encodings.h

uint8 is_bigendian    # is this data bigendian?
uint32 step           # Full row length in bytes
uint8[] data          # actual matrix data, size is (step * rows)

================================================================================
MSG: sensor_msgs/CameraInfo
# This message defines meta information for a camera. It should be in a
# camera namespace on topic \"camera_info\" and accompanied by up to five
# image topics named:
#
#   image_raw - raw data from the camera driver, possibly Bayer encoded
#   image            - monochrome, distorted
#   image_color      - color, distorted
#   image_rect       - monochrome, rectified
#   image_rect_color - color, rectified
#
# The image_pipeline contains packages (image_proc, stereo_image_proc)
# for producing the four processed image topics from image_raw and
# camera_info. The meaning of the camera parameters are described in
# detail at http://www.ros.org/wiki/image_pipeline/CameraInfo.
#
# The image_geometry package provides a user-friendly interface to
# common operations using this meta information. If you want to, e.g.,
# project a 3d point into image coordinates, we strongly recommend
# using image_geometry.
#
# If the camera is uncalibrated, the matrices D, K, R, P should be left
# zeroed out. In particular, clients may assume that K[0] == 0.0
# indicates an uncalibrated camera.

#######################################################################
#                     Image acquisition info                          #
#######################################################################

# Time of image acquisition, camera coordinate frame ID
Header header    # Header timestamp should be acquisition time of image
                 # Header frame_id should be optical frame of camera
                 # origin of frame should be optical center of camera
                 # +x should point to the right in the image
                 # +y should point down in the image
                 # +z should point into the plane of the image


#######################################################################
#                      Calibration Parameters                         #
#######################################################################
# These are fixed during camera calibration. Their values will be the #
# same in all messages until the camera is recalibrated. Note that    #
# self-calibrating systems may \"recalibrate\" frequently.              #
#                                                                     #
# The internal parameters can be used to warp a raw (distorted) image #
# to:                                                                 #
#   1. An undistorted image (requires D and K)                        #
#   2. A rectified image (requires D, K, R)                           #
# The projection matrix P projects 3D points into the rectified image.#
#######################################################################

# The image dimensions with which the camera was calibrated. Normally
# this will be the full camera resolution in pixels.
uint32 height
uint32 width

# The distortion model used. Supported models are listed in
# sensor_msgs/distortion_models.h. For most cameras, \"plumb_bob\" - a
# simple model of radial and tangential distortion - is sufficent.
string distortion_model

# The distortion parameters, size depending on the distortion model.
# For \"plumb_bob\", the 5 parameters are: (k1, k2, t1, t2, k3).
float64[] D

# Intrinsic camera matrix for the raw (distorted) images.
#     [fx  0 cx]
# K = [ 0 fy cy]
#     [ 0  0  1]
# Projects 3D points in the camera coordinate frame to 2D pixel
# coordinates using the focal lengths (fx, fy) and principal point
# (cx, cy).
float64[9]  K # 3x3 row-major matrix

# Rectification matrix (stereo cameras only)
# A rotation matrix aligning the camera coordinate system to the ideal
# stereo image plane so that epipolar lines in both stereo images are
# parallel.
float64[9]  R # 3x3 row-major matrix

# Projection/camera matrix
#     [fx'  0  cx' Tx]
# P = [ 0  fy' cy' Ty]
#     [ 0   0   1   0]
# By convention, this matrix specifies the intrinsic (camera) matrix
#  of the processed (rectified) image. That is, the left 3x3 portion
#  is the normal camera intrinsic matrix for the rectified image.
# It projects 3D points in the camera coordinate frame to 2D pixel
#  coordinates using the focal lengths (fx', fy') and principal point
#  (cx', cy') - these may differ from the values in K.
# For monocular cameras, Tx = Ty = 0. Normally, monocular cameras will
#  also have R = the identity and P[1:3,1:3] = K.
# For a stereo pair, the fourth column [Tx Ty 0]' is related to the
#  position of the optical center of the second camera in the first
#  camera's frame. We assume Tz = 0 so both cameras are in the same
#  stereo image plane. The first camera always has Tx = Ty = 0. For
#  the right (second) camera of a horizontal stereo pair, Ty = 0 and
#  Tx = -fx' * B, where B is the baseline between the cameras.
# Given a 3D point [X Y Z]', the projection (x, y) of the point onto
#  the rectified image is given by:
#  [u v w]' = P * [X Y Z 1]'
#         x = u / w
#         y = v / w
#  This holds for both images of a stereo pair.
float64[12] P # 3x4 row-major matrix


#######################################################################
#                      Operational Parameters                         #
#######################################################################
# These define the image region actually captured by the camera       #
# driver. Although they affect the geometry of the output image, they #
# may be changed freely without recalibrating the camera.             #
#######################################################################

# Binning refers here to any camera setting which combines rectangular
#  neighborhoods of pixels into larger \"super-pixels.\" It reduces the
#  resolution of the output image to
#  (width / binning_x) x (height / binning_y).
# The default values binning_x = binning_y = 0 is considered the same
#  as binning_x = binning_y = 1 (no subsampling).
uint32 binning_x
uint32 binning_y

# Region of interest (subwindow of full camera resolution), given in
#  full resolution (unbinned) image coordinates. A particular ROI
#  always denotes the same window of pixels on the camera sensor,
#  regardless of binning settings.
# The default setting of roi (all values 0) is considered the same as
#  full resolution (roi.width = width, roi.height = height).
RegionOfInterest roi

================================================================================
MSG: sensor_msgs/RegionOfInterest
# This message is used to specify a region of interest within an image.
#
# When used to specify the ROI setting of the camera when the image was
# taken, the height and width fields should either match the height and
# width fields for the associated image; or height = width = 0
# indicates that the full resolution image was captured.

uint32 x_offset  # Leftmost pixel of the ROI
                 # (0 if the ROI includes the left edge of the image)
uint32 y_offset  # Topmost pixel of the ROI
                 # (0 if the ROI includes the top edge of the image)
uint32 height    # Height of ROI
uint32 width     # Width of ROI

# True if a distinct rectified ROI should be calculated from the \"raw\"
# ROI in this message. Typically this should be False if the full image
# is captured (ROI not used), and True if a subwindow is captured (ROI
# used).
bool do_rectify

================================================================================
MSG: geometry_msgs/Vector3
# This represents a vector in free space. 

float64 x
float64 y
float64 z
================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

"
  end
  attr_accessor :graspable_objects, :image, :camera_info, :meshes, :reference_to_camera

  @@struct_L6C = ::ROS::Struct.new("L6C")
  @@struct_LCL = ::ROS::Struct.new("LCL")
  @@struct_C = ::ROS::Struct.new("C")
  @@struct_d12 = ::ROS::Struct.new("d12")
  @@struct_f = ::ROS::Struct.new("f")
  @@struct_l = ::ROS::Struct.new("l")
  @@struct_CL = ::ROS::Struct.new("CL")
  @@struct_f3 = ::ROS::Struct.new("f3")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d7 = ::ROS::Struct.new("d7")
  @@struct_d9 = ::ROS::Struct.new("d9")
  @@struct_CL2 = ::ROS::Struct.new("CL2")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_L4C = ::ROS::Struct.new("L4C")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['manipulation_msgs/GraspableObject[]','sensor_msgs/Image','sensor_msgs/CameraInfo','shape_msgs/Mesh[]','geometry_msgs/Pose']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [manipulation_msgs/GraspableObject[]] :graspable_objects initialize value
  # @option args [sensor_msgs/Image] :image initialize value
  # @option args [sensor_msgs/CameraInfo] :camera_info initialize value
  # @option args [shape_msgs/Mesh[]] :meshes initialize value
  # @option args [geometry_msgs/Pose] :reference_to_camera initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:graspable_objects]
      @graspable_objects = args[:graspable_objects]
    else
      @graspable_objects = []
    end
    if args[:image]
      @image = args[:image]
    else
      @image = Sensor_msgs::Image.new
    end
    if args[:camera_info]
      @camera_info = args[:camera_info]
    else
      @camera_info = Sensor_msgs::CameraInfo.new
    end
    if args[:meshes]
      @meshes = args[:meshes]
    else
      @meshes = []
    end
    if args[:reference_to_camera]
      @reference_to_camera = args[:reference_to_camera]
    else
      @reference_to_camera = Geometry_msgs::Pose.new
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      length = @graspable_objects.length
      buff.write(@@struct_L.pack(length))
      for val1 in @graspable_objects
        _x = val1.reference_frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = val1.potential_models.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.potential_models
          buff.write(@@struct_l.pack(val2.model_id))
          _v4409 = val2.type
          _x = _v4409.key
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = _v4409.db
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v4410 = val2.pose
          _v4411 = _v4410.header
          buff.write(@@struct_L.pack(_v4411.seq))
          _v4412 = _v4411.stamp
          _x = _v4412
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v4411.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v4413 = _v4410.pose
          _v4414 = _v4413.position
          _x = _v4414
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v4415 = _v4413.orientation
          _x = _v4415
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          buff.write(@@struct_f.pack(val2.confidence))
          _x = val2.detector_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
        end
        _v4416 = val1.cluster
        _v4417 = _v4416.header
        buff.write(@@struct_L.pack(_v4417.seq))
        _v4418 = _v4417.stamp
        _x = _v4418
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4417.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = _v4416.points.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v4416.points
          _x = val3
          buff.write(@@struct_f3.pack(_x.x, _x.y, _x.z))
        end
        length = _v4416.channels.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v4416.channels
          _x = val3.name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          length = val3.values.length
          buff.write(@@struct_L.pack(length))
          pattern = "f#{length}"
          buff.write(*val3.values.pack(pattern))
        end
        _v4419 = val1.region
        _v4420 = _v4419.cloud
        _v4421 = _v4420.header
        buff.write(@@struct_L.pack(_v4421.seq))
        _v4422 = _v4421.stamp
        _x = _v4422
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4421.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4420
        buff.write(@@struct_L2.pack(_x.height, _x.width))
        length = _v4420.fields.length
        buff.write(@@struct_L.pack(length))
        for val4 in _v4420.fields
          _x = val4.name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val4
          buff.write(@@struct_LCL.pack(_x.offset, _x.datatype, _x.count))
        end
        _x = _v4420
        buff.write(@@struct_CL2.pack(_x.is_bigendian, _x.point_step, _x.row_step))
        _x = _v4420.data
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        buff.write(@@struct_C.pack(_v4420.is_dense))
        length = _v4419.mask.length
        buff.write(@@struct_L.pack(length))
        pattern = "l#{length}"
        buff.write(*_v4419.mask.pack(pattern))
        _v4423 = _v4419.image
        _v4424 = _v4423.header
        buff.write(@@struct_L.pack(_v4424.seq))
        _v4425 = _v4424.stamp
        _x = _v4425
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4424.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4423
        buff.write(@@struct_L2.pack(_x.height, _x.width))
        _x = _v4423.encoding
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4423
        buff.write(@@struct_CL.pack(_x.is_bigendian, _x.step))
        _x = _v4423.data
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v4426 = _v4419.disparity_image
        _v4427 = _v4426.header
        buff.write(@@struct_L.pack(_v4427.seq))
        _v4428 = _v4427.stamp
        _x = _v4428
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4427.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4426
        buff.write(@@struct_L2.pack(_x.height, _x.width))
        _x = _v4426.encoding
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4426
        buff.write(@@struct_CL.pack(_x.is_bigendian, _x.step))
        _x = _v4426.data
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v4429 = _v4419.cam_info
        _v4430 = _v4429.header
        buff.write(@@struct_L.pack(_v4430.seq))
        _v4431 = _v4430.stamp
        _x = _v4431
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4430.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v4429
        buff.write(@@struct_L2.pack(_x.height, _x.width))
        _x = _v4429.distortion_model
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = _v4429.D.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*_v4429.D.pack(pattern))
        buff.write(@@struct_d9.pack(*_v4429.K))
        buff.write(@@struct_d9.pack(*_v4429.R))
        buff.write(@@struct_d12.pack(*_v4429.P))
        _x = _v4429
        buff.write(@@struct_L2.pack(_x.binning_x, _x.binning_y))
        _v4432 = _v4429.roi
        _x = _v4432
        buff.write(@@struct_L4C.pack(_x.x_offset, _x.y_offset, _x.height, _x.width, _x.do_rectify))
        _v4433 = _v4419.roi_box_pose
        _v4434 = _v4433.header
        buff.write(@@struct_L.pack(_v4434.seq))
        _v4435 = _v4434.stamp
        _x = _v4435
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v4434.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v4436 = _v4433.pose
        _v4437 = _v4436.position
        _x = _v4437
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v4438 = _v4436.orientation
        _x = _v4438
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        _v4439 = _v4419.roi_box_dims
        _x = _v4439
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _x = val1.collision_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
      end
      buff.write(@@struct_L3.pack(@image.header.seq, @image.header.stamp.secs, @image.header.stamp.nsecs))
      _x = @image.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L2.pack(@image.height, @image.width))
      _x = @image.encoding
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_CL.pack(@image.is_bigendian, @image.step))
      _x = @image.data
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L3.pack(@camera_info.header.seq, @camera_info.header.stamp.secs, @camera_info.header.stamp.nsecs))
      _x = @camera_info.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L2.pack(@camera_info.height, @camera_info.width))
      _x = @camera_info.distortion_model
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @camera_info.D.length
      buff.write(@@struct_L.pack(length))
      pattern = "d#{length}"
      buff.write(*@camera_info.D.pack(pattern))
      buff.write(@@struct_d9.pack(*@camera_info.K))
      buff.write(@@struct_d9.pack(*@camera_info.R))
      buff.write(@@struct_d12.pack(*@camera_info.P))
      buff.write(@@struct_L6C.pack(@camera_info.binning_x, @camera_info.binning_y, @camera_info.roi.x_offset, @camera_info.roi.y_offset, @camera_info.roi.height, @camera_info.roi.width, @camera_info.roi.do_rectify))
      length = @meshes.length
      buff.write(@@struct_L.pack(length))
      for val1 in @meshes
        length = val1.triangles.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.triangles
          buff.write(@@struct_L3.pack(*val2.vertex_indices))
        end
        length = val1.vertices.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.vertices
          _x = val2
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        end
      end
      buff.write(@@struct_d7.pack(@reference_to_camera.position.x, @reference_to_camera.position.y, @reference_to_camera.position.z, @reference_to_camera.orientation.x, @reference_to_camera.orientation.y, @reference_to_camera.orientation.z, @reference_to_camera.orientation.w))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @graspable_objects == nil
        @graspable_objects = Manipulation_msgs::GraspableObject[].new
      end
      if @image == nil
        @image = Sensor_msgs::Image.new
      end
      if @camera_info == nil
        @camera_info = Sensor_msgs::CameraInfo.new
      end
      if @meshes == nil
        @meshes = Shape_msgs::Mesh[].new
      end
      if @reference_to_camera == nil
        @reference_to_camera = Geometry_msgs::Pose.new
      end
      end_point = 0
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @graspable_objects = []
      length.times do
        val1 = Manipulation_msgs::GraspableObject.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.reference_frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.potential_models = []
        length.times do
          val2 = Household_objects_database_msgs::DatabaseModelPose.new
          start = end_point
          end_point += ROS::Struct::calc_size('l')
          (val2.model_id,) = @@struct_l.unpack(str[start..(end_point-1)])
          _v4440 = val2.type
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v4440.key = str[start..(end_point-1)]
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v4440.db = str[start..(end_point-1)]
          _v4441 = val2.pose
          _v4442 = _v4441.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v4442.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v4443 = _v4442.stamp
          _x = _v4443
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v4442.frame_id = str[start..(end_point-1)]
          _v4444 = _v4441.pose
          _v4445 = _v4444.position
          _x = _v4445
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v4446 = _v4444.orientation
          _x = _v4446
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += ROS::Struct::calc_size('f')
          (val2.confidence,) = @@struct_f.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.detector_name = str[start..(end_point-1)]
          val1.potential_models.push(val2)
        end
        _v4447 = val1.cluster
        _v4448 = _v4447.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4448.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4449 = _v4448.stamp
        _x = _v4449
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4448.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4447.points = []
        length.times do
          val3 = Geometry_msgs::Point32.new
          _x = val3
          start = end_point
          end_point += ROS::Struct::calc_size('f3')
          (_x.x, _x.y, _x.z,) = @@struct_f3.unpack(str[start..(end_point-1)])
          _v4447.points.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4447.channels = []
        length.times do
          val3 = Sensor_msgs::ChannelFloat32.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val3.name = str[start..(end_point-1)]
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "f#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.values = str[start..(end_point-1)].unpack(pattern)
          _v4447.channels.push(val3)
        end
        _v4450 = val1.region
        _v4451 = _v4450.cloud
        _v4452 = _v4451.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4452.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4453 = _v4452.stamp
        _x = _v4453
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4452.frame_id = str[start..(end_point-1)]
        _x = _v4451
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.height, _x.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4451.fields = []
        length.times do
          val4 = Sensor_msgs::PointField.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val4.name = str[start..(end_point-1)]
          _x = val4
          start = end_point
          end_point += ROS::Struct::calc_size('LCL')
          (_x.offset, _x.datatype, _x.count,) = @@struct_LCL.unpack(str[start..(end_point-1)])
          _v4451.fields.push(val4)
        end
        _x = _v4451
        start = end_point
        end_point += ROS::Struct::calc_size('CL2')
        (_x.is_bigendian, _x.point_step, _x.row_step,) = @@struct_CL2.unpack(str[start..(end_point-1)])
        _v4451.is_bigendian = bool(_v4451.is_bigendian)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4451.data = str[start..(end_point-1)]
        start = end_point
        end_point += ROS::Struct::calc_size('C')
        (_v4451.is_dense,) = @@struct_C.unpack(str[start..(end_point-1)])
        _v4451.is_dense = bool(_v4451.is_dense)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "l#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        _v4450.mask = str[start..(end_point-1)].unpack(pattern)
        _v4454 = _v4450.image
        _v4455 = _v4454.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4455.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4456 = _v4455.stamp
        _x = _v4456
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4455.frame_id = str[start..(end_point-1)]
        _x = _v4454
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.height, _x.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4454.encoding = str[start..(end_point-1)]
        _x = _v4454
        start = end_point
        end_point += ROS::Struct::calc_size('CL')
        (_x.is_bigendian, _x.step,) = @@struct_CL.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4454.data = str[start..(end_point-1)]
        _v4457 = _v4450.disparity_image
        _v4458 = _v4457.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4458.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4459 = _v4458.stamp
        _x = _v4459
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4458.frame_id = str[start..(end_point-1)]
        _x = _v4457
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.height, _x.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4457.encoding = str[start..(end_point-1)]
        _x = _v4457
        start = end_point
        end_point += ROS::Struct::calc_size('CL')
        (_x.is_bigendian, _x.step,) = @@struct_CL.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4457.data = str[start..(end_point-1)]
        _v4460 = _v4450.cam_info
        _v4461 = _v4460.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4461.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4462 = _v4461.stamp
        _x = _v4462
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4461.frame_id = str[start..(end_point-1)]
        _x = _v4460
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.height, _x.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4460.distortion_model = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        _v4460.D = str[start..(end_point-1)].unpack(pattern)
        start = end_point
        end_point += ROS::Struct::calc_size('d9')
        _v4460.K = @@struct_d9.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += ROS::Struct::calc_size('d9')
        _v4460.R = @@struct_d9.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += ROS::Struct::calc_size('d12')
        _v4460.P = @@struct_d12.unpack(str[start..(end_point-1)])
        _x = _v4460
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.binning_x, _x.binning_y,) = @@struct_L2.unpack(str[start..(end_point-1)])
        _v4463 = _v4460.roi
        _x = _v4463
        start = end_point
        end_point += ROS::Struct::calc_size('L4C')
        (_x.x_offset, _x.y_offset, _x.height, _x.width, _x.do_rectify,) = @@struct_L4C.unpack(str[start..(end_point-1)])
        _v4463.do_rectify = bool(_v4463.do_rectify)
        _v4464 = _v4450.roi_box_pose
        _v4465 = _v4464.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v4465.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v4466 = _v4465.stamp
        _x = _v4466
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v4465.frame_id = str[start..(end_point-1)]
        _v4467 = _v4464.pose
        _v4468 = _v4467.position
        _x = _v4468
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v4469 = _v4467.orientation
        _x = _v4469
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        _v4470 = _v4450.roi_box_dims
        _x = _v4470
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.collision_name = str[start..(end_point-1)]
        @graspable_objects.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@image.header.seq, @image.header.stamp.secs, @image.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @image.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L2')
      (@image.height, @image.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @image.encoding = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('CL')
      (@image.is_bigendian, @image.step,) = @@struct_CL.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @image.data = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@camera_info.header.seq, @camera_info.header.stamp.secs, @camera_info.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @camera_info.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L2')
      (@camera_info.height, @camera_info.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @camera_info.distortion_model = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "d#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @camera_info.D = str[start..(end_point-1)].unpack(pattern)
      start = end_point
      end_point += ROS::Struct::calc_size('d9')
      @camera_info.K = @@struct_d9.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('d9')
      @camera_info.R = @@struct_d9.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('d12')
      @camera_info.P = @@struct_d12.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('L6C')
      (@camera_info.binning_x, @camera_info.binning_y, @camera_info.roi.x_offset, @camera_info.roi.y_offset, @camera_info.roi.height, @camera_info.roi.width, @camera_info.roi.do_rectify,) = @@struct_L6C.unpack(str[start..(end_point-1)])
      @camera_info.roi.do_rectify = bool(@camera_info.roi.do_rectify)
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @meshes = []
      length.times do
        val1 = Shape_msgs::Mesh.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.triangles = []
        length.times do
          val2 = Shape_msgs::MeshTriangle.new
          start = end_point
          end_point += ROS::Struct::calc_size('L3')
          val2.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
          val1.triangles.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.vertices = []
        length.times do
          val2 = Geometry_msgs::Point.new
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          val1.vertices.push(val2)
        end
        @meshes.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('d7')
      (@reference_to_camera.position.x, @reference_to_camera.position.y, @reference_to_camera.position.z, @reference_to_camera.orientation.x, @reference_to_camera.orientation.y, @reference_to_camera.orientation.z, @reference_to_camera.orientation.w,) = @@struct_d7.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
