# autogenerated by genrb from AttachedCollisionObject.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "shape_msgs/Plane"
require "trajectory_msgs/JointTrajectory"
require "shape_msgs/SolidPrimitive"
require "trajectory_msgs/JointTrajectoryPoint"
require "ros/duration"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "shape_msgs/Mesh"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Pose"
require "moveit_msgs/CollisionObject"

module Moveit_msgs

class AttachedCollisionObject <::ROS::Message
  def self.md5sum
    "3ceac60b21e85bbd6c5b0ab9d476b752"
  end

  def self.type
    "moveit_msgs/AttachedCollisionObject"
  end

  def has_header?
    false
  end

  def message_definition
    "# The CollisionObject will be attached with a fixed joint to this link
string link_name

#This contains the actual shapes and poses for the CollisionObject
#to be attached to the link
#If action is remove and no object.id is set, all objects
#attached to the link indicated by link_name will be removed
CollisionObject object

# The set of links that the attached objects are allowed to touch
# by default - the link_name is already considered by default
string[] touch_links

# If certain links were placed in a particular posture for this object to remain attached 
# (e.g., an end effector closing around an object), the posture necessary for releasing
# the object is stored here
trajectory_msgs/JointTrajectory detach_posture

# The weight of the attached object, if known
float64 weight

================================================================================
MSG: moveit_msgs/CollisionObject
# a header, used for interpreting the poses
Header header

# the id of the object (name used in MoveIt)
string id

# The object type in a database of known objects
object_recognition_msgs/ObjectType type

# the the collision geometries associated with the object;
# their poses are with respect to the specified header

# solid geometric primitives
shape_msgs/SolidPrimitive[] primitives
geometry_msgs/Pose[] primitive_poses

# meshes
shape_msgs/Mesh[] meshes
geometry_msgs/Pose[] mesh_poses

# bounding planes (equation is specified, but the plane can be oriented using an additional pose)
shape_msgs/Plane[] planes
geometry_msgs/Pose[] plane_poses

# Adds the object to the planning scene. If the object previously existed, it is replaced.
byte ADD=0

# Removes the object from the environment entirely (everything that matches the specified id)
byte REMOVE=1

# Append to an object that already exists in the planning scene. If the does not exist, it is added.
byte APPEND=2

# If an object already exists in the scene, new poses can be sent (the geometry arrays must be left empty)
# if solely moving the object is desired
byte MOVE=3

# Operation to be performed
byte operation

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: shape_msgs/SolidPrimitive
# Define box, sphere, cylinder, cone 
# All shapes are defined to have their bounding boxes centered around 0,0,0.

uint8 BOX=1
uint8 SPHERE=2
uint8 CYLINDER=3
uint8 CONE=4

# The type of the shape
uint8 type


# The dimensions of the shape
float64[] dimensions

# The meaning of the shape dimensions: each constant defines the index in the 'dimensions' array

# For the BOX type, the X, Y, and Z dimensions are the length of the corresponding
# sides of the box.
uint8 BOX_X=0
uint8 BOX_Y=1
uint8 BOX_Z=2


# For the SPHERE type, only one component is used, and it gives the radius of
# the sphere.
uint8 SPHERE_RADIUS=0


# For the CYLINDER and CONE types, the center line is oriented along
# the Z axis.  Therefore the CYLINDER_HEIGHT (CONE_HEIGHT) component
# of dimensions gives the height of the cylinder (cone).  The
# CYLINDER_RADIUS (CONE_RADIUS) component of dimensions gives the
# radius of the base of the cylinder (cone).  Cone and cylinder
# primitives are defined to be circular. The tip of the cone is
# pointing up, along +Z axis.

uint8 CYLINDER_HEIGHT=0
uint8 CYLINDER_RADIUS=1

uint8 CONE_HEIGHT=0
uint8 CONE_RADIUS=1

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: shape_msgs/Plane
# Representation of a plane, using the plane equation ax + by + cz + d = 0

# a := coef[0]
# b := coef[1]
# c := coef[2]
# d := coef[3]

float64[4] coef

================================================================================
MSG: trajectory_msgs/JointTrajectory
Header header
string[] joint_names
JointTrajectoryPoint[] points
================================================================================
MSG: trajectory_msgs/JointTrajectoryPoint
# Each trajectory point specifies either positions[, velocities[, accelerations]]
# or positions[, effort] for the trajectory to be executed.
# All specified values are in the same order as the joint names in JointTrajectory.msg

float64[] positions
float64[] velocities
float64[] accelerations
float64[] effort
duration time_from_start

"
  end
  attr_accessor :link_name, :object, :touch_links, :detach_posture, :weight

  @@struct_C = ::ROS::Struct.new("C")
  @@struct_d = ::ROS::Struct.new("d")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_c = ::ROS::Struct.new("c")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_l2 = ::ROS::Struct.new("l2")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['string','moveit_msgs/CollisionObject','string[]','trajectory_msgs/JointTrajectory','float64']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [string] :link_name initialize value
  # @option args [moveit_msgs/CollisionObject] :object initialize value
  # @option args [string[]] :touch_links initialize value
  # @option args [trajectory_msgs/JointTrajectory] :detach_posture initialize value
  # @option args [float64] :weight initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:link_name]
      @link_name = args[:link_name]
    else
      @link_name = ''
    end
    if args[:object]
      @object = args[:object]
    else
      @object = Moveit_msgs::CollisionObject.new
    end
    if args[:touch_links]
      @touch_links = args[:touch_links]
    else
      @touch_links = []
    end
    if args[:detach_posture]
      @detach_posture = args[:detach_posture]
    else
      @detach_posture = Trajectory_msgs::JointTrajectory.new
    end
    if args[:weight]
      @weight = args[:weight]
    else
      @weight = 0.0
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      _x = @link_name
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L3.pack(@object.header.seq, @object.header.stamp.secs, @object.header.stamp.nsecs))
      _x = @object.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @object.id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @object.type.key
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @object.type.db
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @object.primitives.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.primitives
        buff.write(@@struct_C.pack(val1.type))
        length = val1.dimensions.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.dimensions.pack(pattern))
      end
      length = @object.primitive_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.primitive_poses
        _v371 = val1.position
        _x = _v371
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v372 = val1.orientation
        _x = _v372
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @object.meshes.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.meshes
        length = val1.triangles.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.triangles
          buff.write(@@struct_L3.pack(*val2.vertex_indices))
        end
        length = val1.vertices.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.vertices
          _x = val2
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        end
      end
      length = @object.mesh_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.mesh_poses
        _v373 = val1.position
        _x = _v373
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v374 = val1.orientation
        _x = _v374
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @object.planes.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.planes
        buff.write(@@struct_d4.pack(*val1.coef))
      end
      length = @object.plane_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @object.plane_poses
        _v375 = val1.position
        _x = _v375
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v376 = val1.orientation
        _x = _v376
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      buff.write(@@struct_c.pack(@object.operation))
      length = @touch_links.length
      buff.write(@@struct_L.pack(length))
      for val1 in @touch_links
        length = val1.length
        buff.write([length, val1].pack("La#{length}"))
      end
      buff.write(@@struct_L3.pack(@detach_posture.header.seq, @detach_posture.header.stamp.secs, @detach_posture.header.stamp.nsecs))
      _x = @detach_posture.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @detach_posture.joint_names.length
      buff.write(@@struct_L.pack(length))
      for val1 in @detach_posture.joint_names
        length = val1.length
        buff.write([length, val1].pack("La#{length}"))
      end
      length = @detach_posture.points.length
      buff.write(@@struct_L.pack(length))
      for val1 in @detach_posture.points
        length = val1.positions.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.positions.pack(pattern))
        length = val1.velocities.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.velocities.pack(pattern))
        length = val1.accelerations.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.accelerations.pack(pattern))
        length = val1.effort.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.effort.pack(pattern))
        _v377 = val1.time_from_start
        _x = _v377
        buff.write(@@struct_l2.pack(_x.secs, _x.nsecs))
      end
      buff.write(@@struct_d.pack(@weight))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @object == nil
        @object = Moveit_msgs::CollisionObject.new
      end
      if @detach_posture == nil
        @detach_posture = Trajectory_msgs::JointTrajectory.new
      end
      end_point = 0
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @link_name = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@object.header.seq, @object.header.stamp.secs, @object.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @object.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @object.id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @object.type.key = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @object.type.db = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.primitives = []
      length.times do
        val1 = Shape_msgs::SolidPrimitive.new
        start = end_point
        end_point += ROS::Struct::calc_size('C')
        (val1.type,) = @@struct_C.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.dimensions = str[start..(end_point-1)].unpack(pattern)
        @object.primitives.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.primitive_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v378 = val1.position
        _x = _v378
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v379 = val1.orientation
        _x = _v379
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @object.primitive_poses.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.meshes = []
      length.times do
        val1 = Shape_msgs::Mesh.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.triangles = []
        length.times do
          val2 = Shape_msgs::MeshTriangle.new
          start = end_point
          end_point += ROS::Struct::calc_size('L3')
          val2.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
          val1.triangles.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.vertices = []
        length.times do
          val2 = Geometry_msgs::Point.new
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          val1.vertices.push(val2)
        end
        @object.meshes.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.mesh_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v380 = val1.position
        _x = _v380
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v381 = val1.orientation
        _x = _v381
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @object.mesh_poses.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.planes = []
      length.times do
        val1 = Shape_msgs::Plane.new
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        val1.coef = @@struct_d4.unpack(str[start..(end_point-1)])
        @object.planes.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @object.plane_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v382 = val1.position
        _x = _v382
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v383 = val1.orientation
        _x = _v383
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @object.plane_poses.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('c')
      (@object.operation,) = @@struct_c.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @touch_links = []
      length.times do
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1 = str[start..(end_point-1)]
        @touch_links.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@detach_posture.header.seq, @detach_posture.header.stamp.secs, @detach_posture.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @detach_posture.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @detach_posture.joint_names = []
      length.times do
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1 = str[start..(end_point-1)]
        @detach_posture.joint_names.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @detach_posture.points = []
      length.times do
        val1 = Trajectory_msgs::JointTrajectoryPoint.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.positions = str[start..(end_point-1)].unpack(pattern)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.velocities = str[start..(end_point-1)].unpack(pattern)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.accelerations = str[start..(end_point-1)].unpack(pattern)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.effort = str[start..(end_point-1)].unpack(pattern)
        _v384 = val1.time_from_start
        _x = _v384
        start = end_point
        end_point += ROS::Struct::calc_size('l2')
        (_x.secs, _x.nsecs,) = @@struct_l2.unpack(str[start..(end_point-1)])
        @detach_posture.points.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('d')
      (@weight,) = @@struct_d.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
