# autogenerated by genrb from CollisionObject.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "shape_msgs/Plane"
require "shape_msgs/SolidPrimitive"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "shape_msgs/Mesh"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Pose"

module Moveit_msgs

class CollisionObject <::ROS::Message
  def self.md5sum
    "568a161b26dc46c54a5a07621ce82cf3"
  end

  def self.type
    "moveit_msgs/CollisionObject"
  end

  def has_header?
    true
  end

  def message_definition
    "# a header, used for interpreting the poses
Header header

# the id of the object (name used in MoveIt)
string id

# The object type in a database of known objects
object_recognition_msgs/ObjectType type

# the the collision geometries associated with the object;
# their poses are with respect to the specified header

# solid geometric primitives
shape_msgs/SolidPrimitive[] primitives
geometry_msgs/Pose[] primitive_poses

# meshes
shape_msgs/Mesh[] meshes
geometry_msgs/Pose[] mesh_poses

# bounding planes (equation is specified, but the plane can be oriented using an additional pose)
shape_msgs/Plane[] planes
geometry_msgs/Pose[] plane_poses

# Adds the object to the planning scene. If the object previously existed, it is replaced.
byte ADD=0

# Removes the object from the environment entirely (everything that matches the specified id)
byte REMOVE=1

# Append to an object that already exists in the planning scene. If the does not exist, it is added.
byte APPEND=2

# If an object already exists in the scene, new poses can be sent (the geometry arrays must be left empty)
# if solely moving the object is desired
byte MOVE=3

# Operation to be performed
byte operation

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: shape_msgs/SolidPrimitive
# Define box, sphere, cylinder, cone 
# All shapes are defined to have their bounding boxes centered around 0,0,0.

uint8 BOX=1
uint8 SPHERE=2
uint8 CYLINDER=3
uint8 CONE=4

# The type of the shape
uint8 type


# The dimensions of the shape
float64[] dimensions

# The meaning of the shape dimensions: each constant defines the index in the 'dimensions' array

# For the BOX type, the X, Y, and Z dimensions are the length of the corresponding
# sides of the box.
uint8 BOX_X=0
uint8 BOX_Y=1
uint8 BOX_Z=2


# For the SPHERE type, only one component is used, and it gives the radius of
# the sphere.
uint8 SPHERE_RADIUS=0


# For the CYLINDER and CONE types, the center line is oriented along
# the Z axis.  Therefore the CYLINDER_HEIGHT (CONE_HEIGHT) component
# of dimensions gives the height of the cylinder (cone).  The
# CYLINDER_RADIUS (CONE_RADIUS) component of dimensions gives the
# radius of the base of the cylinder (cone).  Cone and cylinder
# primitives are defined to be circular. The tip of the cone is
# pointing up, along +Z axis.

uint8 CYLINDER_HEIGHT=0
uint8 CYLINDER_RADIUS=1

uint8 CONE_HEIGHT=0
uint8 CONE_RADIUS=1

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: shape_msgs/Plane
# Representation of a plane, using the plane equation ax + by + cz + d = 0

# a := coef[0]
# b := coef[1]
# c := coef[2]
# d := coef[3]

float64[4] coef

"
  end
  # Pseudo-constants
  ADD = 0
  REMOVE = 1
  APPEND = 2
  MOVE = 3

  attr_accessor :header, :id, :type, :primitives, :primitive_poses, :meshes, :mesh_poses, :planes, :plane_poses, :operation

  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_C = ::ROS::Struct.new("C")
  @@struct_c = ::ROS::Struct.new("c")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['std_msgs/Header','string','object_recognition_msgs/ObjectType','shape_msgs/SolidPrimitive[]','geometry_msgs/Pose[]','shape_msgs/Mesh[]','geometry_msgs/Pose[]','shape_msgs/Plane[]','geometry_msgs/Pose[]','byte']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [std_msgs/Header] :header initialize value
  # @option args [string] :id initialize value
  # @option args [object_recognition_msgs/ObjectType] :type initialize value
  # @option args [shape_msgs/SolidPrimitive[]] :primitives initialize value
  # @option args [geometry_msgs/Pose[]] :primitive_poses initialize value
  # @option args [shape_msgs/Mesh[]] :meshes initialize value
  # @option args [geometry_msgs/Pose[]] :mesh_poses initialize value
  # @option args [shape_msgs/Plane[]] :planes initialize value
  # @option args [geometry_msgs/Pose[]] :plane_poses initialize value
  # @option args [byte] :operation initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:header]
      @header = args[:header]
    else
      @header = Std_msgs::Header.new
    end
    if args[:id]
      @id = args[:id]
    else
      @id = ''
    end
    if args[:type]
      @type = args[:type]
    else
      @type = Object_recognition_msgs::ObjectType.new
    end
    if args[:primitives]
      @primitives = args[:primitives]
    else
      @primitives = []
    end
    if args[:primitive_poses]
      @primitive_poses = args[:primitive_poses]
    else
      @primitive_poses = []
    end
    if args[:meshes]
      @meshes = args[:meshes]
    else
      @meshes = []
    end
    if args[:mesh_poses]
      @mesh_poses = args[:mesh_poses]
    else
      @mesh_poses = []
    end
    if args[:planes]
      @planes = args[:planes]
    else
      @planes = []
    end
    if args[:plane_poses]
      @plane_poses = args[:plane_poses]
    else
      @plane_poses = []
    end
    if args[:operation]
      @operation = args[:operation]
    else
      @operation = 0
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_L3.pack(@header.seq, @header.stamp.secs, @header.stamp.nsecs))
      _x = @header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @type.key
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @type.db
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @primitives.length
      buff.write(@@struct_L.pack(length))
      for val1 in @primitives
        buff.write(@@struct_C.pack(val1.type))
        length = val1.dimensions.length
        buff.write(@@struct_L.pack(length))
        pattern = "d#{length}"
        buff.write(*val1.dimensions.pack(pattern))
      end
      length = @primitive_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @primitive_poses
        _v393 = val1.position
        _x = _v393
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v394 = val1.orientation
        _x = _v394
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @meshes.length
      buff.write(@@struct_L.pack(length))
      for val1 in @meshes
        length = val1.triangles.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.triangles
          buff.write(@@struct_L3.pack(*val2.vertex_indices))
        end
        length = val1.vertices.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.vertices
          _x = val2
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        end
      end
      length = @mesh_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @mesh_poses
        _v395 = val1.position
        _x = _v395
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v396 = val1.orientation
        _x = _v396
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @planes.length
      buff.write(@@struct_L.pack(length))
      for val1 in @planes
        buff.write(@@struct_d4.pack(*val1.coef))
      end
      length = @plane_poses.length
      buff.write(@@struct_L.pack(length))
      for val1 in @plane_poses
        _v397 = val1.position
        _x = _v397
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v398 = val1.orientation
        _x = _v398
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      buff.write(@@struct_c.pack(@operation))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @header == nil
        @header = Std_msgs::Header.new
      end
      if @type == nil
        @type = Object_recognition_msgs::ObjectType.new
      end
      if @primitives == nil
        @primitives = Shape_msgs::SolidPrimitive[].new
      end
      if @primitive_poses == nil
        @primitive_poses = Geometry_msgs::Pose[].new
      end
      if @meshes == nil
        @meshes = Shape_msgs::Mesh[].new
      end
      if @mesh_poses == nil
        @mesh_poses = Geometry_msgs::Pose[].new
      end
      if @planes == nil
        @planes = Shape_msgs::Plane[].new
      end
      if @plane_poses == nil
        @plane_poses = Geometry_msgs::Pose[].new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@header.seq, @header.stamp.secs, @header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.key = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.db = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @primitives = []
      length.times do
        val1 = Shape_msgs::SolidPrimitive.new
        start = end_point
        end_point += ROS::Struct::calc_size('C')
        (val1.type,) = @@struct_C.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        pattern = "d#{length}"
        start = end_point
        end_point += ROS::Struct::calc_size("#{pattern}")
        val1.dimensions = str[start..(end_point-1)].unpack(pattern)
        @primitives.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @primitive_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v399 = val1.position
        _x = _v399
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v400 = val1.orientation
        _x = _v400
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @primitive_poses.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @meshes = []
      length.times do
        val1 = Shape_msgs::Mesh.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.triangles = []
        length.times do
          val2 = Shape_msgs::MeshTriangle.new
          start = end_point
          end_point += ROS::Struct::calc_size('L3')
          val2.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
          val1.triangles.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.vertices = []
        length.times do
          val2 = Geometry_msgs::Point.new
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          val1.vertices.push(val2)
        end
        @meshes.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @mesh_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v401 = val1.position
        _x = _v401
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v402 = val1.orientation
        _x = _v402
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @mesh_poses.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @planes = []
      length.times do
        val1 = Shape_msgs::Plane.new
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        val1.coef = @@struct_d4.unpack(str[start..(end_point-1)])
        @planes.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @plane_poses = []
      length.times do
        val1 = Geometry_msgs::Pose.new
        _v403 = val1.position
        _x = _v403
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v404 = val1.orientation
        _x = _v404
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @plane_poses.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('c')
      (@operation,) = @@struct_c.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
