# autogenerated by genrb from Constraints.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "shape_msgs/SolidPrimitive"
require "moveit_msgs/VisibilityConstraint"
require "geometry_msgs/PoseStamped"
require "moveit_msgs/BoundingVolume"
require "moveit_msgs/JointConstraint"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "moveit_msgs/OrientationConstraint"
require "shape_msgs/Mesh"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Vector3"
require "geometry_msgs/Pose"
require "moveit_msgs/PositionConstraint"

module Moveit_msgs

class Constraints <::ROS::Message
  def self.md5sum
    "8d5ce8d34ef26c65fb5d43c9e99bf6e0"
  end

  def self.type
    "moveit_msgs/Constraints"
  end

  def has_header?
    false
  end

  def message_definition
    "# This message contains a list of motion planning constraints.
# All constraints must be satisfied for a goal to be considered valid

string name

JointConstraint[] joint_constraints

PositionConstraint[] position_constraints

OrientationConstraint[] orientation_constraints

VisibilityConstraint[] visibility_constraints

================================================================================
MSG: moveit_msgs/JointConstraint
# Constrain the position of a joint to be within a certain bound
string joint_name

# the bound to be achieved is [position - tolerance_below, position + tolerance_above]
float64 position
float64 tolerance_above
float64 tolerance_below

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight
================================================================================
MSG: moveit_msgs/PositionConstraint
# This message contains the definition of a position constraint.

Header header

# The robot link this constraint refers to
string link_name

# The offset (in the link frame) for the target point on the link we are planning for
geometry_msgs/Vector3 target_point_offset

# The volume this constraint refers to 
BoundingVolume constraint_region

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Vector3
# This represents a vector in free space. 

float64 x
float64 y
float64 z
================================================================================
MSG: moveit_msgs/BoundingVolume
# Define a volume in 3D

# A set of solid geometric primitives that make up the volume to define (as a union)
shape_msgs/SolidPrimitive[] primitives

# The poses at which the primitives are located
geometry_msgs/Pose[] primitive_poses

# In addition to primitives, meshes can be specified to add to the bounding volume (again, as union)
shape_msgs/Mesh[] meshes

# The poses at which the meshes are located
geometry_msgs/Pose[] mesh_poses

================================================================================
MSG: shape_msgs/SolidPrimitive
# Define box, sphere, cylinder, cone 
# All shapes are defined to have their bounding boxes centered around 0,0,0.

uint8 BOX=1
uint8 SPHERE=2
uint8 CYLINDER=3
uint8 CONE=4

# The type of the shape
uint8 type


# The dimensions of the shape
float64[] dimensions

# The meaning of the shape dimensions: each constant defines the index in the 'dimensions' array

# For the BOX type, the X, Y, and Z dimensions are the length of the corresponding
# sides of the box.
uint8 BOX_X=0
uint8 BOX_Y=1
uint8 BOX_Z=2


# For the SPHERE type, only one component is used, and it gives the radius of
# the sphere.
uint8 SPHERE_RADIUS=0


# For the CYLINDER and CONE types, the center line is oriented along
# the Z axis.  Therefore the CYLINDER_HEIGHT (CONE_HEIGHT) component
# of dimensions gives the height of the cylinder (cone).  The
# CYLINDER_RADIUS (CONE_RADIUS) component of dimensions gives the
# radius of the base of the cylinder (cone).  Cone and cylinder
# primitives are defined to be circular. The tip of the cone is
# pointing up, along +Z axis.

uint8 CYLINDER_HEIGHT=0
uint8 CYLINDER_RADIUS=1

uint8 CONE_HEIGHT=0
uint8 CONE_RADIUS=1

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: moveit_msgs/OrientationConstraint
# This message contains the definition of an orientation constraint.

Header header

# The desired orientation of the robot link specified as a quaternion
geometry_msgs/Quaternion orientation

# The robot link this constraint refers to
string link_name

# optional axis-angle error tolerances specified
float64 absolute_x_axis_tolerance
float64 absolute_y_axis_tolerance
float64 absolute_z_axis_tolerance

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: moveit_msgs/VisibilityConstraint
# The constraint is useful to maintain visibility to a disc (the target) in a particular frame.
# This disc forms the base of a visibiliy cone whose tip is at the origin of the sensor.
# Maintaining visibility is done by ensuring the robot does not obstruct the visibility cone.
# Note:
# This constraint does NOT enforce minimum or maximum distances between the sensor
# and the target, nor does it enforce the target to be in the field of view of
# the sensor. A PositionConstraint can (and probably should) be used for such purposes.

# The radius of the disc that should be maintained visible 
float64 target_radius

# The pose of the disc; as the robot moves, the pose of the disc may change as well
# This can be in the frame of a particular robot link, for example
geometry_msgs/PoseStamped target_pose

# From the sensor origin towards the target, the disc forms a visibility cone
# This cone is approximated using many sides. For example, when using 4 sides, 
# that in fact makes the visibility region be a pyramid.
# This value should always be 3 or more.
int32 cone_sides

# The pose in which visibility is to be maintained.
# The frame id should represent the robot link to which the sensor is attached.
# It is assumed the sensor can look directly at the target, in any direction.
# This assumption is usually not true, but additional PositionConstraints
# can resolve this issue.
geometry_msgs/PoseStamped sensor_pose

# Even though the disc is maintained visible, the visibility cone can be very small
# because of the orientation of the disc with respect to the sensor. It is possible
# for example to view the disk almost from a side, in which case the visibility cone 
# can end up having close to 0 volume. The view angle is defined to be the angle between
# the normal to the visibility disc and the direction vector from the sensor origin.
# The value below represents the minimum desired view angle. For a perfect view,
# this value will be 0 (the two vectors are exact opposites). For a completely obstructed view
# this value will be Pi/2 (the vectors are perpendicular). This value defined below 
# is the maximum view angle to be maintained. This should be a value in the open interval
# (0, Pi/2). If 0 is set, the view angle is NOT enforced.
float64 max_view_angle

# This angle is used similarly to max_view_angle but limits the maximum angle
# between the sensor origin direction vector and the axis that connects the
# sensor origin to the target frame origin. The value is again in the range (0, Pi/2)
# and is NOT enforced if set to 0.
float64 max_range_angle

# The axis that is assumed to indicate the direction of view for the sensor
# X = 2, Y = 1, Z = 0
uint8 SENSOR_Z=0
uint8 SENSOR_Y=1
uint8 SENSOR_X=2
uint8 sensor_view_direction

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: geometry_msgs/PoseStamped
# A Pose with reference coordinate frame and timestamp
Header header
Pose pose

"
  end
  attr_accessor :name, :joint_constraints, :position_constraints, :orientation_constraints, :visibility_constraints

  @@struct_C = ::ROS::Struct.new("C")
  @@struct_d = ::ROS::Struct.new("d")
  @@struct_l = ::ROS::Struct.new("l")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_d2Cd = ::ROS::Struct.new("d2Cd")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['string','moveit_msgs/JointConstraint[]','moveit_msgs/PositionConstraint[]','moveit_msgs/OrientationConstraint[]','moveit_msgs/VisibilityConstraint[]']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [string] :name initialize value
  # @option args [moveit_msgs/JointConstraint[]] :joint_constraints initialize value
  # @option args [moveit_msgs/PositionConstraint[]] :position_constraints initialize value
  # @option args [moveit_msgs/OrientationConstraint[]] :orientation_constraints initialize value
  # @option args [moveit_msgs/VisibilityConstraint[]] :visibility_constraints initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:name]
      @name = args[:name]
    else
      @name = ''
    end
    if args[:joint_constraints]
      @joint_constraints = args[:joint_constraints]
    else
      @joint_constraints = []
    end
    if args[:position_constraints]
      @position_constraints = args[:position_constraints]
    else
      @position_constraints = []
    end
    if args[:orientation_constraints]
      @orientation_constraints = args[:orientation_constraints]
    else
      @orientation_constraints = []
    end
    if args[:visibility_constraints]
      @visibility_constraints = args[:visibility_constraints]
    else
      @visibility_constraints = []
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      _x = @name
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @joint_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @joint_constraints
        _x = val1.joint_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1
        buff.write(@@struct_d4.pack(_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight))
      end
      length = @position_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @position_constraints
        _v405 = val1.header
        buff.write(@@struct_L.pack(_v405.seq))
        _v406 = _v405.stamp
        _x = _v406
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v405.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1.link_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v407 = val1.target_point_offset
        _x = _v407
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v408 = val1.constraint_region
        length = _v408.primitives.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v408.primitives
          buff.write(@@struct_C.pack(val3.type))
          length = val3.dimensions.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.dimensions.pack(pattern))
        end
        length = _v408.primitive_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v408.primitive_poses
          _v409 = val3.position
          _x = _v409
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v410 = val3.orientation
          _x = _v410
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = _v408.meshes.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v408.meshes
          length = val3.triangles.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.triangles
            buff.write(@@struct_L3.pack(*val4.vertex_indices))
          end
          length = val3.vertices.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.vertices
            _x = val4
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          end
        end
        length = _v408.mesh_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v408.mesh_poses
          _v411 = val3.position
          _x = _v411
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v412 = val3.orientation
          _x = _v412
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        buff.write(@@struct_d.pack(val1.weight))
      end
      length = @orientation_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @orientation_constraints
        _v413 = val1.header
        buff.write(@@struct_L.pack(_v413.seq))
        _v414 = _v413.stamp
        _x = _v414
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v413.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v415 = val1.orientation
        _x = _v415
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        _x = val1.link_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1
        buff.write(@@struct_d4.pack(_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight))
      end
      length = @visibility_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @visibility_constraints
        buff.write(@@struct_d.pack(val1.target_radius))
        _v416 = val1.target_pose
        _v417 = _v416.header
        buff.write(@@struct_L.pack(_v417.seq))
        _v418 = _v417.stamp
        _x = _v418
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v417.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v419 = _v416.pose
        _v420 = _v419.position
        _x = _v420
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v421 = _v419.orientation
        _x = _v421
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        buff.write(@@struct_l.pack(val1.cone_sides))
        _v422 = val1.sensor_pose
        _v423 = _v422.header
        buff.write(@@struct_L.pack(_v423.seq))
        _v424 = _v423.stamp
        _x = _v424
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v423.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v425 = _v422.pose
        _v426 = _v425.position
        _x = _v426
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v427 = _v425.orientation
        _x = _v427
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        _x = val1
        buff.write(@@struct_d2Cd.pack(_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight))
      end
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @joint_constraints == nil
        @joint_constraints = Moveit_msgs::JointConstraint[].new
      end
      if @position_constraints == nil
        @position_constraints = Moveit_msgs::PositionConstraint[].new
      end
      if @orientation_constraints == nil
        @orientation_constraints = Moveit_msgs::OrientationConstraint[].new
      end
      if @visibility_constraints == nil
        @visibility_constraints = Moveit_msgs::VisibilityConstraint[].new
      end
      end_point = 0
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @name = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @joint_constraints = []
      length.times do
        val1 = Moveit_msgs::JointConstraint.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.joint_name = str[start..(end_point-1)]
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @joint_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @position_constraints = []
      length.times do
        val1 = Moveit_msgs::PositionConstraint.new
        _v428 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v428.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v429 = _v428.stamp
        _x = _v429
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v428.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.link_name = str[start..(end_point-1)]
        _v430 = val1.target_point_offset
        _x = _v430
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v431 = val1.constraint_region
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v431.primitives = []
        length.times do
          val3 = Shape_msgs::SolidPrimitive.new
          start = end_point
          end_point += ROS::Struct::calc_size('C')
          (val3.type,) = @@struct_C.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.dimensions = str[start..(end_point-1)].unpack(pattern)
          _v431.primitives.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v431.primitive_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v432 = val3.position
          _x = _v432
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v433 = val3.orientation
          _x = _v433
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v431.primitive_poses.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v431.meshes = []
        length.times do
          val3 = Shape_msgs::Mesh.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.triangles = []
          length.times do
            val4 = Shape_msgs::MeshTriangle.new
            start = end_point
            end_point += ROS::Struct::calc_size('L3')
            val4.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
            val3.triangles.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.vertices = []
          length.times do
            val4 = Geometry_msgs::Point.new
            _x = val4
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            val3.vertices.push(val4)
          end
          _v431.meshes.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v431.mesh_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v434 = val3.position
          _x = _v434
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v435 = val3.orientation
          _x = _v435
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v431.mesh_poses.push(val3)
        end
        start = end_point
        end_point += ROS::Struct::calc_size('d')
        (val1.weight,) = @@struct_d.unpack(str[start..(end_point-1)])
        @position_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @orientation_constraints = []
      length.times do
        val1 = Moveit_msgs::OrientationConstraint.new
        _v436 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v436.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v437 = _v436.stamp
        _x = _v437
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v436.frame_id = str[start..(end_point-1)]
        _v438 = val1.orientation
        _x = _v438
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.link_name = str[start..(end_point-1)]
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @orientation_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @visibility_constraints = []
      length.times do
        val1 = Moveit_msgs::VisibilityConstraint.new
        start = end_point
        end_point += ROS::Struct::calc_size('d')
        (val1.target_radius,) = @@struct_d.unpack(str[start..(end_point-1)])
        _v439 = val1.target_pose
        _v440 = _v439.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v440.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v441 = _v440.stamp
        _x = _v441
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v440.frame_id = str[start..(end_point-1)]
        _v442 = _v439.pose
        _v443 = _v442.position
        _x = _v443
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v444 = _v442.orientation
        _x = _v444
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += ROS::Struct::calc_size('l')
        (val1.cone_sides,) = @@struct_l.unpack(str[start..(end_point-1)])
        _v445 = val1.sensor_pose
        _v446 = _v445.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v446.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v447 = _v446.stamp
        _x = _v447
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v446.frame_id = str[start..(end_point-1)]
        _v448 = _v445.pose
        _v449 = _v448.position
        _x = _v449
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v450 = _v448.orientation
        _x = _v450
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d2Cd')
        (_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight,) = @@struct_d2Cd.unpack(str[start..(end_point-1)])
        @visibility_constraints.push(val1)
      end
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
