# autogenerated by genrb from MotionPlanRequest.msg. Do not edit.
require 'ros/message'

require "geometry_msgs/Transform"
require "moveit_msgs/VisibilityConstraint"
require "moveit_msgs/OrientationConstraint"
require "moveit_msgs/Constraints"
require "geometry_msgs/Point"
require "geometry_msgs/Twist"
require "shape_msgs/Mesh"
require "moveit_msgs/PositionConstraint"
require "std_msgs/Header"
require "geometry_msgs/PoseStamped"
require "ros/duration"
require "moveit_msgs/BoundingVolume"
require "moveit_msgs/JointConstraint"
require "shape_msgs/Plane"
require "sensor_msgs/MultiDOFJointState"
require "geometry_msgs/Pose"
require "shape_msgs/MeshTriangle"
require "moveit_msgs/TrajectoryConstraints"
require "shape_msgs/SolidPrimitive"
require "geometry_msgs/Wrench"
require "moveit_msgs/WorkspaceParameters"
require "moveit_msgs/RobotState"
require "trajectory_msgs/JointTrajectory"
require "sensor_msgs/JointState"
require "trajectory_msgs/JointTrajectoryPoint"
require "object_recognition_msgs/ObjectType"
require "moveit_msgs/AttachedCollisionObject"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Vector3"
require "moveit_msgs/CollisionObject"

module Moveit_msgs

class MotionPlanRequest <::ROS::Message
  def self.md5sum
    "7cd790e04c3a55f6742ec387a72a02d6"
  end

  def self.type
    "moveit_msgs/MotionPlanRequest"
  end

  def has_header?
    false
  end

  def message_definition
    "# This service contains the definition for a request to the motion
# planner and the output it provides

# Parameters for the workspace that the planner should work inside
WorkspaceParameters workspace_parameters

# Starting state updates. If certain joints should be considered
# at positions other than the current ones, these positions should
# be set here
RobotState start_state

# The possible goal states for the model to plan for. Each element of
# the array defines a goal region. The goal is achieved
# if the constraints for a particular region are satisfied
Constraints[] goal_constraints

# No state at any point along the path in the produced motion plan will violate these constraints (this applies to all points, not just waypoints)
Constraints path_constraints

# The constraints the resulting trajectory must satisfy
TrajectoryConstraints trajectory_constraints

# The name of the motion planner to use. If no name is specified,
# a default motion planner will be used
string planner_id

# The name of the group of joints on which this planner is operating
string group_name

# The number of times this plan is to be computed. Shortest solution
# will be reported.
int32 num_planning_attempts

# The maximum amount of time the motion planner is allowed to plan for (in seconds)
float64 allowed_planning_time

================================================================================
MSG: moveit_msgs/WorkspaceParameters
# This message contains a set of parameters useful in
# setting up the volume (a box) in which the robot is allowed to move.
# This is useful only when planning for mobile parts of 
# the robot as well.

# Define the frame of reference for the box corners
Header header

# The minumum corner of the box, with respect to the robot starting pose
geometry_msgs/Vector3 min_corner

# The maximum corner of the box, with respect to the robot starting pose
geometry_msgs/Vector3 max_corner

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Vector3
# This represents a vector in free space. 

float64 x
float64 y
float64 z
================================================================================
MSG: moveit_msgs/RobotState
# This message contains information about the robot state, i.e. the positions of its joints and links
sensor_msgs/JointState joint_state

# Joints that may have multiple DOF are specified here
sensor_msgs/MultiDOFJointState multi_dof_joint_state

# Attached collision objects (attached to some link on the robot)
AttachedCollisionObject[] attached_collision_objects

# Flag indicating whether this scene is to be interpreted as a diff with respect to some other scene
# This is mostly important for handling the attached bodies (whether or not to clear the attached bodies
# of a moveit::core::RobotState before updating it with this message)
bool is_diff

================================================================================
MSG: sensor_msgs/JointState
# This is a message that holds data to describe the state of a set of torque controlled joints. 
#
# The state of each joint (revolute or prismatic) is defined by:
#  * the position of the joint (rad or m),
#  * the velocity of the joint (rad/s or m/s) and 
#  * the effort that is applied in the joint (Nm or N).
#
# Each joint is uniquely identified by its name
# The header specifies the time at which the joint states were recorded. All the joint states
# in one message have to be recorded at the same time.
#
# This message consists of a multiple arrays, one for each part of the joint state. 
# The goal is to make each of the fields optional. When e.g. your joints have no
# effort associated with them, you can leave the effort array empty. 
#
# All arrays in this message should have the same size, or be empty.
# This is the only way to uniquely associate the joint name with the correct
# states.


Header header

string[] name
float64[] position
float64[] velocity
float64[] effort

================================================================================
MSG: sensor_msgs/MultiDOFJointState
# Representation of state for joints with multiple degrees of freedom, 
# following the structure of JointState.
#
# It is assumed that a joint in a system corresponds to a transform that gets applied 
# along the kinematic chain. For example, a planar joint (as in URDF) is 3DOF (x, y, yaw)
# and those 3DOF can be expressed as a transformation matrix, and that transformation
# matrix can be converted back to (x, y, yaw)
#
# Each joint is uniquely identified by its name
# The header specifies the time at which the joint states were recorded. All the joint states
# in one message have to be recorded at the same time.
#
# This message consists of a multiple arrays, one for each part of the joint state. 
# The goal is to make each of the fields optional. When e.g. your joints have no
# wrench associated with them, you can leave the wrench array empty. 
#
# All arrays in this message should have the same size, or be empty.
# This is the only way to uniquely associate the joint name with the correct
# states.

Header header

string[] joint_names
geometry_msgs/Transform[] transforms
geometry_msgs/Twist[] twist
geometry_msgs/Wrench[] wrench

================================================================================
MSG: geometry_msgs/Transform
# This represents the transform between two coordinate frames in free space.

Vector3 translation
Quaternion rotation

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: geometry_msgs/Twist
# This expresses velocity in free space broken into its linear and angular parts.
Vector3  linear
Vector3  angular

================================================================================
MSG: geometry_msgs/Wrench
# This represents force in free space, separated into
# its linear and angular parts.
Vector3  force
Vector3  torque

================================================================================
MSG: moveit_msgs/AttachedCollisionObject
# The CollisionObject will be attached with a fixed joint to this link
string link_name

#This contains the actual shapes and poses for the CollisionObject
#to be attached to the link
#If action is remove and no object.id is set, all objects
#attached to the link indicated by link_name will be removed
CollisionObject object

# The set of links that the attached objects are allowed to touch
# by default - the link_name is already considered by default
string[] touch_links

# If certain links were placed in a particular posture for this object to remain attached 
# (e.g., an end effector closing around an object), the posture necessary for releasing
# the object is stored here
trajectory_msgs/JointTrajectory detach_posture

# The weight of the attached object, if known
float64 weight

================================================================================
MSG: moveit_msgs/CollisionObject
# a header, used for interpreting the poses
Header header

# the id of the object (name used in MoveIt)
string id

# The object type in a database of known objects
object_recognition_msgs/ObjectType type

# the the collision geometries associated with the object;
# their poses are with respect to the specified header

# solid geometric primitives
shape_msgs/SolidPrimitive[] primitives
geometry_msgs/Pose[] primitive_poses

# meshes
shape_msgs/Mesh[] meshes
geometry_msgs/Pose[] mesh_poses

# bounding planes (equation is specified, but the plane can be oriented using an additional pose)
shape_msgs/Plane[] planes
geometry_msgs/Pose[] plane_poses

# Adds the object to the planning scene. If the object previously existed, it is replaced.
byte ADD=0

# Removes the object from the environment entirely (everything that matches the specified id)
byte REMOVE=1

# Append to an object that already exists in the planning scene. If the does not exist, it is added.
byte APPEND=2

# If an object already exists in the scene, new poses can be sent (the geometry arrays must be left empty)
# if solely moving the object is desired
byte MOVE=3

# Operation to be performed
byte operation

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: shape_msgs/SolidPrimitive
# Define box, sphere, cylinder, cone 
# All shapes are defined to have their bounding boxes centered around 0,0,0.

uint8 BOX=1
uint8 SPHERE=2
uint8 CYLINDER=3
uint8 CONE=4

# The type of the shape
uint8 type


# The dimensions of the shape
float64[] dimensions

# The meaning of the shape dimensions: each constant defines the index in the 'dimensions' array

# For the BOX type, the X, Y, and Z dimensions are the length of the corresponding
# sides of the box.
uint8 BOX_X=0
uint8 BOX_Y=1
uint8 BOX_Z=2


# For the SPHERE type, only one component is used, and it gives the radius of
# the sphere.
uint8 SPHERE_RADIUS=0


# For the CYLINDER and CONE types, the center line is oriented along
# the Z axis.  Therefore the CYLINDER_HEIGHT (CONE_HEIGHT) component
# of dimensions gives the height of the cylinder (cone).  The
# CYLINDER_RADIUS (CONE_RADIUS) component of dimensions gives the
# radius of the base of the cylinder (cone).  Cone and cylinder
# primitives are defined to be circular. The tip of the cone is
# pointing up, along +Z axis.

uint8 CYLINDER_HEIGHT=0
uint8 CYLINDER_RADIUS=1

uint8 CONE_HEIGHT=0
uint8 CONE_RADIUS=1

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: shape_msgs/Plane
# Representation of a plane, using the plane equation ax + by + cz + d = 0

# a := coef[0]
# b := coef[1]
# c := coef[2]
# d := coef[3]

float64[4] coef

================================================================================
MSG: trajectory_msgs/JointTrajectory
Header header
string[] joint_names
JointTrajectoryPoint[] points
================================================================================
MSG: trajectory_msgs/JointTrajectoryPoint
# Each trajectory point specifies either positions[, velocities[, accelerations]]
# or positions[, effort] for the trajectory to be executed.
# All specified values are in the same order as the joint names in JointTrajectory.msg

float64[] positions
float64[] velocities
float64[] accelerations
float64[] effort
duration time_from_start

================================================================================
MSG: moveit_msgs/Constraints
# This message contains a list of motion planning constraints.
# All constraints must be satisfied for a goal to be considered valid

string name

JointConstraint[] joint_constraints

PositionConstraint[] position_constraints

OrientationConstraint[] orientation_constraints

VisibilityConstraint[] visibility_constraints

================================================================================
MSG: moveit_msgs/JointConstraint
# Constrain the position of a joint to be within a certain bound
string joint_name

# the bound to be achieved is [position - tolerance_below, position + tolerance_above]
float64 position
float64 tolerance_above
float64 tolerance_below

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight
================================================================================
MSG: moveit_msgs/PositionConstraint
# This message contains the definition of a position constraint.

Header header

# The robot link this constraint refers to
string link_name

# The offset (in the link frame) for the target point on the link we are planning for
geometry_msgs/Vector3 target_point_offset

# The volume this constraint refers to 
BoundingVolume constraint_region

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: moveit_msgs/BoundingVolume
# Define a volume in 3D

# A set of solid geometric primitives that make up the volume to define (as a union)
shape_msgs/SolidPrimitive[] primitives

# The poses at which the primitives are located
geometry_msgs/Pose[] primitive_poses

# In addition to primitives, meshes can be specified to add to the bounding volume (again, as union)
shape_msgs/Mesh[] meshes

# The poses at which the meshes are located
geometry_msgs/Pose[] mesh_poses

================================================================================
MSG: moveit_msgs/OrientationConstraint
# This message contains the definition of an orientation constraint.

Header header

# The desired orientation of the robot link specified as a quaternion
geometry_msgs/Quaternion orientation

# The robot link this constraint refers to
string link_name

# optional axis-angle error tolerances specified
float64 absolute_x_axis_tolerance
float64 absolute_y_axis_tolerance
float64 absolute_z_axis_tolerance

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: moveit_msgs/VisibilityConstraint
# The constraint is useful to maintain visibility to a disc (the target) in a particular frame.
# This disc forms the base of a visibiliy cone whose tip is at the origin of the sensor.
# Maintaining visibility is done by ensuring the robot does not obstruct the visibility cone.
# Note:
# This constraint does NOT enforce minimum or maximum distances between the sensor
# and the target, nor does it enforce the target to be in the field of view of
# the sensor. A PositionConstraint can (and probably should) be used for such purposes.

# The radius of the disc that should be maintained visible 
float64 target_radius

# The pose of the disc; as the robot moves, the pose of the disc may change as well
# This can be in the frame of a particular robot link, for example
geometry_msgs/PoseStamped target_pose

# From the sensor origin towards the target, the disc forms a visibility cone
# This cone is approximated using many sides. For example, when using 4 sides, 
# that in fact makes the visibility region be a pyramid.
# This value should always be 3 or more.
int32 cone_sides

# The pose in which visibility is to be maintained.
# The frame id should represent the robot link to which the sensor is attached.
# It is assumed the sensor can look directly at the target, in any direction.
# This assumption is usually not true, but additional PositionConstraints
# can resolve this issue.
geometry_msgs/PoseStamped sensor_pose

# Even though the disc is maintained visible, the visibility cone can be very small
# because of the orientation of the disc with respect to the sensor. It is possible
# for example to view the disk almost from a side, in which case the visibility cone 
# can end up having close to 0 volume. The view angle is defined to be the angle between
# the normal to the visibility disc and the direction vector from the sensor origin.
# The value below represents the minimum desired view angle. For a perfect view,
# this value will be 0 (the two vectors are exact opposites). For a completely obstructed view
# this value will be Pi/2 (the vectors are perpendicular). This value defined below 
# is the maximum view angle to be maintained. This should be a value in the open interval
# (0, Pi/2). If 0 is set, the view angle is NOT enforced.
float64 max_view_angle

# This angle is used similarly to max_view_angle but limits the maximum angle
# between the sensor origin direction vector and the axis that connects the
# sensor origin to the target frame origin. The value is again in the range (0, Pi/2)
# and is NOT enforced if set to 0.
float64 max_range_angle

# The axis that is assumed to indicate the direction of view for the sensor
# X = 2, Y = 1, Z = 0
uint8 SENSOR_Z=0
uint8 SENSOR_Y=1
uint8 SENSOR_X=2
uint8 sensor_view_direction

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: geometry_msgs/PoseStamped
# A Pose with reference coordinate frame and timestamp
Header header
Pose pose

================================================================================
MSG: moveit_msgs/TrajectoryConstraints
# The array of constraints to consider along the trajectory
Constraints[] constraints

"
  end
  attr_accessor :workspace_parameters, :start_state, :goal_constraints, :path_constraints, :trajectory_constraints, :planner_id, :group_name, :num_planning_attempts, :allowed_planning_time

  @@struct_C = ::ROS::Struct.new("C")
  @@struct_d = ::ROS::Struct.new("d")
  @@struct_d6L3 = ::ROS::Struct.new("d6L3")
  @@struct_l2 = ::ROS::Struct.new("l2")
  @@struct_l = ::ROS::Struct.new("l")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_c = ::ROS::Struct.new("c")
  @@struct_ld = ::ROS::Struct.new("ld")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_d2Cd = ::ROS::Struct.new("d2Cd")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['moveit_msgs/WorkspaceParameters','moveit_msgs/RobotState','moveit_msgs/Constraints[]','moveit_msgs/Constraints','moveit_msgs/TrajectoryConstraints','string','string','int32','float64']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [moveit_msgs/WorkspaceParameters] :workspace_parameters initialize value
  # @option args [moveit_msgs/RobotState] :start_state initialize value
  # @option args [moveit_msgs/Constraints[]] :goal_constraints initialize value
  # @option args [moveit_msgs/Constraints] :path_constraints initialize value
  # @option args [moveit_msgs/TrajectoryConstraints] :trajectory_constraints initialize value
  # @option args [string] :planner_id initialize value
  # @option args [string] :group_name initialize value
  # @option args [int32] :num_planning_attempts initialize value
  # @option args [float64] :allowed_planning_time initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:workspace_parameters]
      @workspace_parameters = args[:workspace_parameters]
    else
      @workspace_parameters = Moveit_msgs::WorkspaceParameters.new
    end
    if args[:start_state]
      @start_state = args[:start_state]
    else
      @start_state = Moveit_msgs::RobotState.new
    end
    if args[:goal_constraints]
      @goal_constraints = args[:goal_constraints]
    else
      @goal_constraints = []
    end
    if args[:path_constraints]
      @path_constraints = args[:path_constraints]
    else
      @path_constraints = Moveit_msgs::Constraints.new
    end
    if args[:trajectory_constraints]
      @trajectory_constraints = args[:trajectory_constraints]
    else
      @trajectory_constraints = Moveit_msgs::TrajectoryConstraints.new
    end
    if args[:planner_id]
      @planner_id = args[:planner_id]
    else
      @planner_id = ''
    end
    if args[:group_name]
      @group_name = args[:group_name]
    else
      @group_name = ''
    end
    if args[:num_planning_attempts]
      @num_planning_attempts = args[:num_planning_attempts]
    else
      @num_planning_attempts = 0
    end
    if args[:allowed_planning_time]
      @allowed_planning_time = args[:allowed_planning_time]
    else
      @allowed_planning_time = 0.0
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_L3.pack(@workspace_parameters.header.seq, @workspace_parameters.header.stamp.secs, @workspace_parameters.header.stamp.nsecs))
      _x = @workspace_parameters.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d6L3.pack(@workspace_parameters.min_corner.x, @workspace_parameters.min_corner.y, @workspace_parameters.min_corner.z, @workspace_parameters.max_corner.x, @workspace_parameters.max_corner.y, @workspace_parameters.max_corner.z, @start_state.joint_state.header.seq, @start_state.joint_state.header.stamp.secs, @start_state.joint_state.header.stamp.nsecs))
      _x = @start_state.joint_state.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @start_state.joint_state.name.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.joint_state.name
        length = val1.length
        buff.write([length, val1].pack("La#{length}"))
      end
      length = @start_state.joint_state.position.length
      buff.write(@@struct_L.pack(length))
      pattern = "d#{length}"
      buff.write(*@start_state.joint_state.position.pack(pattern))
      length = @start_state.joint_state.velocity.length
      buff.write(@@struct_L.pack(length))
      pattern = "d#{length}"
      buff.write(*@start_state.joint_state.velocity.pack(pattern))
      length = @start_state.joint_state.effort.length
      buff.write(@@struct_L.pack(length))
      pattern = "d#{length}"
      buff.write(*@start_state.joint_state.effort.pack(pattern))
      buff.write(@@struct_L3.pack(@start_state.multi_dof_joint_state.header.seq, @start_state.multi_dof_joint_state.header.stamp.secs, @start_state.multi_dof_joint_state.header.stamp.nsecs))
      _x = @start_state.multi_dof_joint_state.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @start_state.multi_dof_joint_state.joint_names.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.multi_dof_joint_state.joint_names
        length = val1.length
        buff.write([length, val1].pack("La#{length}"))
      end
      length = @start_state.multi_dof_joint_state.transforms.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.multi_dof_joint_state.transforms
        _v633 = val1.translation
        _x = _v633
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v634 = val1.rotation
        _x = _v634
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
      end
      length = @start_state.multi_dof_joint_state.twist.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.multi_dof_joint_state.twist
        _v635 = val1.linear
        _x = _v635
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v636 = val1.angular
        _x = _v636
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
      length = @start_state.multi_dof_joint_state.wrench.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.multi_dof_joint_state.wrench
        _v637 = val1.force
        _x = _v637
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v638 = val1.torque
        _x = _v638
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
      length = @start_state.attached_collision_objects.length
      buff.write(@@struct_L.pack(length))
      for val1 in @start_state.attached_collision_objects
        _x = val1.link_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v639 = val1.object
        _v640 = _v639.header
        buff.write(@@struct_L.pack(_v640.seq))
        _v641 = _v640.stamp
        _x = _v641
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v640.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v639.id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v642 = _v639.type
        _x = _v642.key
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v642.db
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = _v639.primitives.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.primitives
          buff.write(@@struct_C.pack(val3.type))
          length = val3.dimensions.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.dimensions.pack(pattern))
        end
        length = _v639.primitive_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.primitive_poses
          _v643 = val3.position
          _x = _v643
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v644 = val3.orientation
          _x = _v644
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = _v639.meshes.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.meshes
          length = val3.triangles.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.triangles
            buff.write(@@struct_L3.pack(*val4.vertex_indices))
          end
          length = val3.vertices.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.vertices
            _x = val4
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          end
        end
        length = _v639.mesh_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.mesh_poses
          _v645 = val3.position
          _x = _v645
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v646 = val3.orientation
          _x = _v646
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = _v639.planes.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.planes
          buff.write(@@struct_d4.pack(*val3.coef))
        end
        length = _v639.plane_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v639.plane_poses
          _v647 = val3.position
          _x = _v647
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v648 = val3.orientation
          _x = _v648
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        buff.write(@@struct_c.pack(_v639.operation))
        length = val1.touch_links.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.touch_links
          length = val2.length
          buff.write([length, val2].pack("La#{length}"))
        end
        _v649 = val1.detach_posture
        _v650 = _v649.header
        buff.write(@@struct_L.pack(_v650.seq))
        _v651 = _v650.stamp
        _x = _v651
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v650.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = _v649.joint_names.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v649.joint_names
          length = val3.length
          buff.write([length, val3].pack("La#{length}"))
        end
        length = _v649.points.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v649.points
          length = val3.positions.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.positions.pack(pattern))
          length = val3.velocities.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.velocities.pack(pattern))
          length = val3.accelerations.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.accelerations.pack(pattern))
          length = val3.effort.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.effort.pack(pattern))
          _v652 = val3.time_from_start
          _x = _v652
          buff.write(@@struct_l2.pack(_x.secs, _x.nsecs))
        end
        buff.write(@@struct_d.pack(val1.weight))
      end
      buff.write(@@struct_C.pack(@start_state.is_diff))
      length = @goal_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @goal_constraints
        _x = val1.name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = val1.joint_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.joint_constraints
          _x = val2.joint_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2
          buff.write(@@struct_d4.pack(_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight))
        end
        length = val1.position_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.position_constraints
          _v653 = val2.header
          buff.write(@@struct_L.pack(_v653.seq))
          _v654 = _v653.stamp
          _x = _v654
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v653.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2.link_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v655 = val2.target_point_offset
          _x = _v655
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v656 = val2.constraint_region
          length = _v656.primitives.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v656.primitives
            buff.write(@@struct_C.pack(val4.type))
            length = val4.dimensions.length
            buff.write(@@struct_L.pack(length))
            pattern = "d#{length}"
            buff.write(*val4.dimensions.pack(pattern))
          end
          length = _v656.primitive_poses.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v656.primitive_poses
            _v657 = val4.position
            _x = _v657
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            _v658 = val4.orientation
            _x = _v658
            buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          end
          length = _v656.meshes.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v656.meshes
            length = val4.triangles.length
            buff.write(@@struct_L.pack(length))
            for val5 in val4.triangles
              buff.write(@@struct_L3.pack(*val5.vertex_indices))
            end
            length = val4.vertices.length
            buff.write(@@struct_L.pack(length))
            for val5 in val4.vertices
              _x = val5
              buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            end
          end
          length = _v656.mesh_poses.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v656.mesh_poses
            _v659 = val4.position
            _x = _v659
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            _v660 = val4.orientation
            _x = _v660
            buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          end
          buff.write(@@struct_d.pack(val2.weight))
        end
        length = val1.orientation_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.orientation_constraints
          _v661 = val2.header
          buff.write(@@struct_L.pack(_v661.seq))
          _v662 = _v661.stamp
          _x = _v662
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v661.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v663 = val2.orientation
          _x = _v663
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          _x = val2.link_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2
          buff.write(@@struct_d4.pack(_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight))
        end
        length = val1.visibility_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.visibility_constraints
          buff.write(@@struct_d.pack(val2.target_radius))
          _v664 = val2.target_pose
          _v665 = _v664.header
          buff.write(@@struct_L.pack(_v665.seq))
          _v666 = _v665.stamp
          _x = _v666
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v665.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v667 = _v664.pose
          _v668 = _v667.position
          _x = _v668
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v669 = _v667.orientation
          _x = _v669
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          buff.write(@@struct_l.pack(val2.cone_sides))
          _v670 = val2.sensor_pose
          _v671 = _v670.header
          buff.write(@@struct_L.pack(_v671.seq))
          _v672 = _v671.stamp
          _x = _v672
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v671.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v673 = _v670.pose
          _v674 = _v673.position
          _x = _v674
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v675 = _v673.orientation
          _x = _v675
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          _x = val2
          buff.write(@@struct_d2Cd.pack(_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight))
        end
      end
      _x = @path_constraints.name
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @path_constraints.joint_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @path_constraints.joint_constraints
        _x = val1.joint_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1
        buff.write(@@struct_d4.pack(_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight))
      end
      length = @path_constraints.position_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @path_constraints.position_constraints
        _v676 = val1.header
        buff.write(@@struct_L.pack(_v676.seq))
        _v677 = _v676.stamp
        _x = _v677
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v676.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1.link_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v678 = val1.target_point_offset
        _x = _v678
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v679 = val1.constraint_region
        length = _v679.primitives.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v679.primitives
          buff.write(@@struct_C.pack(val3.type))
          length = val3.dimensions.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val3.dimensions.pack(pattern))
        end
        length = _v679.primitive_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v679.primitive_poses
          _v680 = val3.position
          _x = _v680
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v681 = val3.orientation
          _x = _v681
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = _v679.meshes.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v679.meshes
          length = val3.triangles.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.triangles
            buff.write(@@struct_L3.pack(*val4.vertex_indices))
          end
          length = val3.vertices.length
          buff.write(@@struct_L.pack(length))
          for val4 in val3.vertices
            _x = val4
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          end
        end
        length = _v679.mesh_poses.length
        buff.write(@@struct_L.pack(length))
        for val3 in _v679.mesh_poses
          _v682 = val3.position
          _x = _v682
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v683 = val3.orientation
          _x = _v683
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        buff.write(@@struct_d.pack(val1.weight))
      end
      length = @path_constraints.orientation_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @path_constraints.orientation_constraints
        _v684 = val1.header
        buff.write(@@struct_L.pack(_v684.seq))
        _v685 = _v684.stamp
        _x = _v685
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v684.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v686 = val1.orientation
        _x = _v686
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        _x = val1.link_name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1
        buff.write(@@struct_d4.pack(_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight))
      end
      length = @path_constraints.visibility_constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @path_constraints.visibility_constraints
        buff.write(@@struct_d.pack(val1.target_radius))
        _v687 = val1.target_pose
        _v688 = _v687.header
        buff.write(@@struct_L.pack(_v688.seq))
        _v689 = _v688.stamp
        _x = _v689
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v688.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v690 = _v687.pose
        _v691 = _v690.position
        _x = _v691
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v692 = _v690.orientation
        _x = _v692
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        buff.write(@@struct_l.pack(val1.cone_sides))
        _v693 = val1.sensor_pose
        _v694 = _v693.header
        buff.write(@@struct_L.pack(_v694.seq))
        _v695 = _v694.stamp
        _x = _v695
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v694.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v696 = _v693.pose
        _v697 = _v696.position
        _x = _v697
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
        _v698 = _v696.orientation
        _x = _v698
        buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        _x = val1
        buff.write(@@struct_d2Cd.pack(_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight))
      end
      length = @trajectory_constraints.constraints.length
      buff.write(@@struct_L.pack(length))
      for val1 in @trajectory_constraints.constraints
        _x = val1.name
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = val1.joint_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.joint_constraints
          _x = val2.joint_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2
          buff.write(@@struct_d4.pack(_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight))
        end
        length = val1.position_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.position_constraints
          _v699 = val2.header
          buff.write(@@struct_L.pack(_v699.seq))
          _v700 = _v699.stamp
          _x = _v700
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v699.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2.link_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v701 = val2.target_point_offset
          _x = _v701
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v702 = val2.constraint_region
          length = _v702.primitives.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v702.primitives
            buff.write(@@struct_C.pack(val4.type))
            length = val4.dimensions.length
            buff.write(@@struct_L.pack(length))
            pattern = "d#{length}"
            buff.write(*val4.dimensions.pack(pattern))
          end
          length = _v702.primitive_poses.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v702.primitive_poses
            _v703 = val4.position
            _x = _v703
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            _v704 = val4.orientation
            _x = _v704
            buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          end
          length = _v702.meshes.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v702.meshes
            length = val4.triangles.length
            buff.write(@@struct_L.pack(length))
            for val5 in val4.triangles
              buff.write(@@struct_L3.pack(*val5.vertex_indices))
            end
            length = val4.vertices.length
            buff.write(@@struct_L.pack(length))
            for val5 in val4.vertices
              _x = val5
              buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            end
          end
          length = _v702.mesh_poses.length
          buff.write(@@struct_L.pack(length))
          for val4 in _v702.mesh_poses
            _v705 = val4.position
            _x = _v705
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
            _v706 = val4.orientation
            _x = _v706
            buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          end
          buff.write(@@struct_d.pack(val2.weight))
        end
        length = val1.orientation_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.orientation_constraints
          _v707 = val2.header
          buff.write(@@struct_L.pack(_v707.seq))
          _v708 = _v707.stamp
          _x = _v708
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v707.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v709 = val2.orientation
          _x = _v709
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          _x = val2.link_name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2
          buff.write(@@struct_d4.pack(_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight))
        end
        length = val1.visibility_constraints.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.visibility_constraints
          buff.write(@@struct_d.pack(val2.target_radius))
          _v710 = val2.target_pose
          _v711 = _v710.header
          buff.write(@@struct_L.pack(_v711.seq))
          _v712 = _v711.stamp
          _x = _v712
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v711.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v713 = _v710.pose
          _v714 = _v713.position
          _x = _v714
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v715 = _v713.orientation
          _x = _v715
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          buff.write(@@struct_l.pack(val2.cone_sides))
          _v716 = val2.sensor_pose
          _v717 = _v716.header
          buff.write(@@struct_L.pack(_v717.seq))
          _v718 = _v717.stamp
          _x = _v718
          buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
          _x = _v717.frame_id
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _v719 = _v716.pose
          _v720 = _v719.position
          _x = _v720
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v721 = _v719.orientation
          _x = _v721
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
          _x = val2
          buff.write(@@struct_d2Cd.pack(_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight))
        end
      end
      _x = @planner_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @group_name
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_ld.pack(@num_planning_attempts, @allowed_planning_time))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @workspace_parameters == nil
        @workspace_parameters = Moveit_msgs::WorkspaceParameters.new
      end
      if @start_state == nil
        @start_state = Moveit_msgs::RobotState.new
      end
      if @goal_constraints == nil
        @goal_constraints = Moveit_msgs::Constraints[].new
      end
      if @path_constraints == nil
        @path_constraints = Moveit_msgs::Constraints.new
      end
      if @trajectory_constraints == nil
        @trajectory_constraints = Moveit_msgs::TrajectoryConstraints.new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@workspace_parameters.header.seq, @workspace_parameters.header.stamp.secs, @workspace_parameters.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @workspace_parameters.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d6L3')
      (@workspace_parameters.min_corner.x, @workspace_parameters.min_corner.y, @workspace_parameters.min_corner.z, @workspace_parameters.max_corner.x, @workspace_parameters.max_corner.y, @workspace_parameters.max_corner.z, @start_state.joint_state.header.seq, @start_state.joint_state.header.stamp.secs, @start_state.joint_state.header.stamp.nsecs,) = @@struct_d6L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @start_state.joint_state.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.joint_state.name = []
      length.times do
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1 = str[start..(end_point-1)]
        @start_state.joint_state.name.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "d#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @start_state.joint_state.position = str[start..(end_point-1)].unpack(pattern)
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "d#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @start_state.joint_state.velocity = str[start..(end_point-1)].unpack(pattern)
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "d#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @start_state.joint_state.effort = str[start..(end_point-1)].unpack(pattern)
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@start_state.multi_dof_joint_state.header.seq, @start_state.multi_dof_joint_state.header.stamp.secs, @start_state.multi_dof_joint_state.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @start_state.multi_dof_joint_state.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.multi_dof_joint_state.joint_names = []
      length.times do
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1 = str[start..(end_point-1)]
        @start_state.multi_dof_joint_state.joint_names.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.multi_dof_joint_state.transforms = []
      length.times do
        val1 = Geometry_msgs::Transform.new
        _v722 = val1.translation
        _x = _v722
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v723 = val1.rotation
        _x = _v723
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @start_state.multi_dof_joint_state.transforms.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.multi_dof_joint_state.twist = []
      length.times do
        val1 = Geometry_msgs::Twist.new
        _v724 = val1.linear
        _x = _v724
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v725 = val1.angular
        _x = _v725
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @start_state.multi_dof_joint_state.twist.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.multi_dof_joint_state.wrench = []
      length.times do
        val1 = Geometry_msgs::Wrench.new
        _v726 = val1.force
        _x = _v726
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v727 = val1.torque
        _x = _v727
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @start_state.multi_dof_joint_state.wrench.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @start_state.attached_collision_objects = []
      length.times do
        val1 = Moveit_msgs::AttachedCollisionObject.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.link_name = str[start..(end_point-1)]
        _v728 = val1.object
        _v729 = _v728.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v729.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v730 = _v729.stamp
        _x = _v730
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v729.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v728.id = str[start..(end_point-1)]
        _v731 = _v728.type
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v731.key = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v731.db = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.primitives = []
        length.times do
          val3 = Shape_msgs::SolidPrimitive.new
          start = end_point
          end_point += ROS::Struct::calc_size('C')
          (val3.type,) = @@struct_C.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.dimensions = str[start..(end_point-1)].unpack(pattern)
          _v728.primitives.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.primitive_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v732 = val3.position
          _x = _v732
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v733 = val3.orientation
          _x = _v733
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v728.primitive_poses.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.meshes = []
        length.times do
          val3 = Shape_msgs::Mesh.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.triangles = []
          length.times do
            val4 = Shape_msgs::MeshTriangle.new
            start = end_point
            end_point += ROS::Struct::calc_size('L3')
            val4.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
            val3.triangles.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.vertices = []
          length.times do
            val4 = Geometry_msgs::Point.new
            _x = val4
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            val3.vertices.push(val4)
          end
          _v728.meshes.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.mesh_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v734 = val3.position
          _x = _v734
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v735 = val3.orientation
          _x = _v735
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v728.mesh_poses.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.planes = []
        length.times do
          val3 = Shape_msgs::Plane.new
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          val3.coef = @@struct_d4.unpack(str[start..(end_point-1)])
          _v728.planes.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v728.plane_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v736 = val3.position
          _x = _v736
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v737 = val3.orientation
          _x = _v737
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v728.plane_poses.push(val3)
        end
        start = end_point
        end_point += ROS::Struct::calc_size('c')
        (_v728.operation,) = @@struct_c.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.touch_links = []
        length.times do
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2 = str[start..(end_point-1)]
          val1.touch_links.push(val2)
        end
        _v738 = val1.detach_posture
        _v739 = _v738.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v739.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v740 = _v739.stamp
        _x = _v740
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v739.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v738.joint_names = []
        length.times do
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val3 = str[start..(end_point-1)]
          _v738.joint_names.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v738.points = []
        length.times do
          val3 = Trajectory_msgs::JointTrajectoryPoint.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.positions = str[start..(end_point-1)].unpack(pattern)
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.velocities = str[start..(end_point-1)].unpack(pattern)
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.accelerations = str[start..(end_point-1)].unpack(pattern)
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.effort = str[start..(end_point-1)].unpack(pattern)
          _v741 = val3.time_from_start
          _x = _v741
          start = end_point
          end_point += ROS::Struct::calc_size('l2')
          (_x.secs, _x.nsecs,) = @@struct_l2.unpack(str[start..(end_point-1)])
          _v738.points.push(val3)
        end
        start = end_point
        end_point += ROS::Struct::calc_size('d')
        (val1.weight,) = @@struct_d.unpack(str[start..(end_point-1)])
        @start_state.attached_collision_objects.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('C')
      (@start_state.is_diff,) = @@struct_C.unpack(str[start..(end_point-1)])
      @start_state.is_diff = bool(@start_state.is_diff)
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @goal_constraints = []
      length.times do
        val1 = Moveit_msgs::Constraints.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.name = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.joint_constraints = []
        length.times do
          val2 = Moveit_msgs::JointConstraint.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.joint_name = str[start..(end_point-1)]
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.joint_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.position_constraints = []
        length.times do
          val2 = Moveit_msgs::PositionConstraint.new
          _v742 = val2.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v742.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v743 = _v742.stamp
          _x = _v743
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v742.frame_id = str[start..(end_point-1)]
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.link_name = str[start..(end_point-1)]
          _v744 = val2.target_point_offset
          _x = _v744
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v745 = val2.constraint_region
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v745.primitives = []
          length.times do
            val4 = Shape_msgs::SolidPrimitive.new
            start = end_point
            end_point += ROS::Struct::calc_size('C')
            (val4.type,) = @@struct_C.unpack(str[start..(end_point-1)])
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            pattern = "d#{length}"
            start = end_point
            end_point += ROS::Struct::calc_size("#{pattern}")
            val4.dimensions = str[start..(end_point-1)].unpack(pattern)
            _v745.primitives.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v745.primitive_poses = []
          length.times do
            val4 = Geometry_msgs::Pose.new
            _v746 = val4.position
            _x = _v746
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            _v747 = val4.orientation
            _x = _v747
            start = end_point
            end_point += ROS::Struct::calc_size('d4')
            (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
            _v745.primitive_poses.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v745.meshes = []
          length.times do
            val4 = Shape_msgs::Mesh.new
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            val4.triangles = []
            length.times do
              val5 = Shape_msgs::MeshTriangle.new
              start = end_point
              end_point += ROS::Struct::calc_size('L3')
              val5.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
              val4.triangles.push(val5)
            end
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            val4.vertices = []
            length.times do
              val5 = Geometry_msgs::Point.new
              _x = val5
              start = end_point
              end_point += ROS::Struct::calc_size('d3')
              (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
              val4.vertices.push(val5)
            end
            _v745.meshes.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v745.mesh_poses = []
          length.times do
            val4 = Geometry_msgs::Pose.new
            _v748 = val4.position
            _x = _v748
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            _v749 = val4.orientation
            _x = _v749
            start = end_point
            end_point += ROS::Struct::calc_size('d4')
            (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
            _v745.mesh_poses.push(val4)
          end
          start = end_point
          end_point += ROS::Struct::calc_size('d')
          (val2.weight,) = @@struct_d.unpack(str[start..(end_point-1)])
          val1.position_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.orientation_constraints = []
        length.times do
          val2 = Moveit_msgs::OrientationConstraint.new
          _v750 = val2.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v750.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v751 = _v750.stamp
          _x = _v751
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v750.frame_id = str[start..(end_point-1)]
          _v752 = val2.orientation
          _x = _v752
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.link_name = str[start..(end_point-1)]
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.orientation_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.visibility_constraints = []
        length.times do
          val2 = Moveit_msgs::VisibilityConstraint.new
          start = end_point
          end_point += ROS::Struct::calc_size('d')
          (val2.target_radius,) = @@struct_d.unpack(str[start..(end_point-1)])
          _v753 = val2.target_pose
          _v754 = _v753.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v754.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v755 = _v754.stamp
          _x = _v755
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v754.frame_id = str[start..(end_point-1)]
          _v756 = _v753.pose
          _v757 = _v756.position
          _x = _v757
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v758 = _v756.orientation
          _x = _v758
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += ROS::Struct::calc_size('l')
          (val2.cone_sides,) = @@struct_l.unpack(str[start..(end_point-1)])
          _v759 = val2.sensor_pose
          _v760 = _v759.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v760.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v761 = _v760.stamp
          _x = _v761
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v760.frame_id = str[start..(end_point-1)]
          _v762 = _v759.pose
          _v763 = _v762.position
          _x = _v763
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v764 = _v762.orientation
          _x = _v764
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d2Cd')
          (_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight,) = @@struct_d2Cd.unpack(str[start..(end_point-1)])
          val1.visibility_constraints.push(val2)
        end
        @goal_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @path_constraints.name = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @path_constraints.joint_constraints = []
      length.times do
        val1 = Moveit_msgs::JointConstraint.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.joint_name = str[start..(end_point-1)]
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @path_constraints.joint_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @path_constraints.position_constraints = []
      length.times do
        val1 = Moveit_msgs::PositionConstraint.new
        _v765 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v765.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v766 = _v765.stamp
        _x = _v766
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v765.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.link_name = str[start..(end_point-1)]
        _v767 = val1.target_point_offset
        _x = _v767
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v768 = val1.constraint_region
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v768.primitives = []
        length.times do
          val3 = Shape_msgs::SolidPrimitive.new
          start = end_point
          end_point += ROS::Struct::calc_size('C')
          (val3.type,) = @@struct_C.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val3.dimensions = str[start..(end_point-1)].unpack(pattern)
          _v768.primitives.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v768.primitive_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v769 = val3.position
          _x = _v769
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v770 = val3.orientation
          _x = _v770
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v768.primitive_poses.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v768.meshes = []
        length.times do
          val3 = Shape_msgs::Mesh.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.triangles = []
          length.times do
            val4 = Shape_msgs::MeshTriangle.new
            start = end_point
            end_point += ROS::Struct::calc_size('L3')
            val4.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
            val3.triangles.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val3.vertices = []
          length.times do
            val4 = Geometry_msgs::Point.new
            _x = val4
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            val3.vertices.push(val4)
          end
          _v768.meshes.push(val3)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v768.mesh_poses = []
        length.times do
          val3 = Geometry_msgs::Pose.new
          _v771 = val3.position
          _x = _v771
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v772 = val3.orientation
          _x = _v772
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _v768.mesh_poses.push(val3)
        end
        start = end_point
        end_point += ROS::Struct::calc_size('d')
        (val1.weight,) = @@struct_d.unpack(str[start..(end_point-1)])
        @path_constraints.position_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @path_constraints.orientation_constraints = []
      length.times do
        val1 = Moveit_msgs::OrientationConstraint.new
        _v773 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v773.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v774 = _v773.stamp
        _x = _v774
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v773.frame_id = str[start..(end_point-1)]
        _v775 = val1.orientation
        _x = _v775
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.link_name = str[start..(end_point-1)]
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
        @path_constraints.orientation_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @path_constraints.visibility_constraints = []
      length.times do
        val1 = Moveit_msgs::VisibilityConstraint.new
        start = end_point
        end_point += ROS::Struct::calc_size('d')
        (val1.target_radius,) = @@struct_d.unpack(str[start..(end_point-1)])
        _v776 = val1.target_pose
        _v777 = _v776.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v777.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v778 = _v777.stamp
        _x = _v778
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v777.frame_id = str[start..(end_point-1)]
        _v779 = _v776.pose
        _v780 = _v779.position
        _x = _v780
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v781 = _v779.orientation
        _x = _v781
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += ROS::Struct::calc_size('l')
        (val1.cone_sides,) = @@struct_l.unpack(str[start..(end_point-1)])
        _v782 = val1.sensor_pose
        _v783 = _v782.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v783.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v784 = _v783.stamp
        _x = _v784
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v783.frame_id = str[start..(end_point-1)]
        _v785 = _v782.pose
        _v786 = _v785.position
        _x = _v786
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        _v787 = _v785.orientation
        _x = _v787
        start = end_point
        end_point += ROS::Struct::calc_size('d4')
        (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d2Cd')
        (_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight,) = @@struct_d2Cd.unpack(str[start..(end_point-1)])
        @path_constraints.visibility_constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @trajectory_constraints.constraints = []
      length.times do
        val1 = Moveit_msgs::Constraints.new
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.name = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.joint_constraints = []
        length.times do
          val2 = Moveit_msgs::JointConstraint.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.joint_name = str[start..(end_point-1)]
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.position, _x.tolerance_above, _x.tolerance_below, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.joint_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.position_constraints = []
        length.times do
          val2 = Moveit_msgs::PositionConstraint.new
          _v788 = val2.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v788.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v789 = _v788.stamp
          _x = _v789
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v788.frame_id = str[start..(end_point-1)]
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.link_name = str[start..(end_point-1)]
          _v790 = val2.target_point_offset
          _x = _v790
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v791 = val2.constraint_region
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v791.primitives = []
          length.times do
            val4 = Shape_msgs::SolidPrimitive.new
            start = end_point
            end_point += ROS::Struct::calc_size('C')
            (val4.type,) = @@struct_C.unpack(str[start..(end_point-1)])
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            pattern = "d#{length}"
            start = end_point
            end_point += ROS::Struct::calc_size("#{pattern}")
            val4.dimensions = str[start..(end_point-1)].unpack(pattern)
            _v791.primitives.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v791.primitive_poses = []
          length.times do
            val4 = Geometry_msgs::Pose.new
            _v792 = val4.position
            _x = _v792
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            _v793 = val4.orientation
            _x = _v793
            start = end_point
            end_point += ROS::Struct::calc_size('d4')
            (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
            _v791.primitive_poses.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v791.meshes = []
          length.times do
            val4 = Shape_msgs::Mesh.new
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            val4.triangles = []
            length.times do
              val5 = Shape_msgs::MeshTriangle.new
              start = end_point
              end_point += ROS::Struct::calc_size('L3')
              val5.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
              val4.triangles.push(val5)
            end
            start = end_point
            end_point += 4
            (length,) = @@struct_L.unpack(str[start..(end_point-1)])
            val4.vertices = []
            length.times do
              val5 = Geometry_msgs::Point.new
              _x = val5
              start = end_point
              end_point += ROS::Struct::calc_size('d3')
              (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
              val4.vertices.push(val5)
            end
            _v791.meshes.push(val4)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v791.mesh_poses = []
          length.times do
            val4 = Geometry_msgs::Pose.new
            _v794 = val4.position
            _x = _v794
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            _v795 = val4.orientation
            _x = _v795
            start = end_point
            end_point += ROS::Struct::calc_size('d4')
            (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
            _v791.mesh_poses.push(val4)
          end
          start = end_point
          end_point += ROS::Struct::calc_size('d')
          (val2.weight,) = @@struct_d.unpack(str[start..(end_point-1)])
          val1.position_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.orientation_constraints = []
        length.times do
          val2 = Moveit_msgs::OrientationConstraint.new
          _v796 = val2.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v796.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v797 = _v796.stamp
          _x = _v797
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v796.frame_id = str[start..(end_point-1)]
          _v798 = val2.orientation
          _x = _v798
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.link_name = str[start..(end_point-1)]
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.absolute_x_axis_tolerance, _x.absolute_y_axis_tolerance, _x.absolute_z_axis_tolerance, _x.weight,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.orientation_constraints.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.visibility_constraints = []
        length.times do
          val2 = Moveit_msgs::VisibilityConstraint.new
          start = end_point
          end_point += ROS::Struct::calc_size('d')
          (val2.target_radius,) = @@struct_d.unpack(str[start..(end_point-1)])
          _v799 = val2.target_pose
          _v800 = _v799.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v800.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v801 = _v800.stamp
          _x = _v801
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v800.frame_id = str[start..(end_point-1)]
          _v802 = _v799.pose
          _v803 = _v802.position
          _x = _v803
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v804 = _v802.orientation
          _x = _v804
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += ROS::Struct::calc_size('l')
          (val2.cone_sides,) = @@struct_l.unpack(str[start..(end_point-1)])
          _v805 = val2.sensor_pose
          _v806 = _v805.header
          start = end_point
          end_point += ROS::Struct::calc_size('L')
          (_v806.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
          _v807 = _v806.stamp
          _x = _v807
          start = end_point
          end_point += ROS::Struct::calc_size('L2')
          (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          _v806.frame_id = str[start..(end_point-1)]
          _v808 = _v805.pose
          _v809 = _v808.position
          _x = _v809
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v810 = _v808.orientation
          _x = _v810
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('d2Cd')
          (_x.max_view_angle, _x.max_range_angle, _x.sensor_view_direction, _x.weight,) = @@struct_d2Cd.unpack(str[start..(end_point-1)])
          val1.visibility_constraints.push(val2)
        end
        @trajectory_constraints.constraints.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @planner_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @group_name = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('ld')
      (@num_planning_attempts, @allowed_planning_time,) = @@struct_ld.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
