# autogenerated by genrb from PlanningSceneWorld.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "shape_msgs/Plane"
require "shape_msgs/SolidPrimitive"
require "octomap_msgs/Octomap"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "shape_msgs/Mesh"
require "geometry_msgs/Quaternion"
require "octomap_msgs/OctomapWithPose"
require "geometry_msgs/Pose"
require "moveit_msgs/CollisionObject"

module Moveit_msgs

class PlanningSceneWorld <::ROS::Message
  def self.md5sum
    "373d88390d1db385335639f687723ee6"
  end

  def self.type
    "moveit_msgs/PlanningSceneWorld"
  end

  def has_header?
    false
  end

  def message_definition
    "# collision objects
CollisionObject[] collision_objects

# The octomap that represents additional collision data
octomap_msgs/OctomapWithPose octomap

================================================================================
MSG: moveit_msgs/CollisionObject
# a header, used for interpreting the poses
Header header

# the id of the object (name used in MoveIt)
string id

# The object type in a database of known objects
object_recognition_msgs/ObjectType type

# the the collision geometries associated with the object;
# their poses are with respect to the specified header

# solid geometric primitives
shape_msgs/SolidPrimitive[] primitives
geometry_msgs/Pose[] primitive_poses

# meshes
shape_msgs/Mesh[] meshes
geometry_msgs/Pose[] mesh_poses

# bounding planes (equation is specified, but the plane can be oriented using an additional pose)
shape_msgs/Plane[] planes
geometry_msgs/Pose[] plane_poses

# Adds the object to the planning scene. If the object previously existed, it is replaced.
byte ADD=0

# Removes the object from the environment entirely (everything that matches the specified id)
byte REMOVE=1

# Append to an object that already exists in the planning scene. If the does not exist, it is added.
byte APPEND=2

# If an object already exists in the scene, new poses can be sent (the geometry arrays must be left empty)
# if solely moving the object is desired
byte MOVE=3

# Operation to be performed
byte operation

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: shape_msgs/SolidPrimitive
# Define box, sphere, cylinder, cone 
# All shapes are defined to have their bounding boxes centered around 0,0,0.

uint8 BOX=1
uint8 SPHERE=2
uint8 CYLINDER=3
uint8 CONE=4

# The type of the shape
uint8 type


# The dimensions of the shape
float64[] dimensions

# The meaning of the shape dimensions: each constant defines the index in the 'dimensions' array

# For the BOX type, the X, Y, and Z dimensions are the length of the corresponding
# sides of the box.
uint8 BOX_X=0
uint8 BOX_Y=1
uint8 BOX_Z=2


# For the SPHERE type, only one component is used, and it gives the radius of
# the sphere.
uint8 SPHERE_RADIUS=0


# For the CYLINDER and CONE types, the center line is oriented along
# the Z axis.  Therefore the CYLINDER_HEIGHT (CONE_HEIGHT) component
# of dimensions gives the height of the cylinder (cone).  The
# CYLINDER_RADIUS (CONE_RADIUS) component of dimensions gives the
# radius of the base of the cylinder (cone).  Cone and cylinder
# primitives are defined to be circular. The tip of the cone is
# pointing up, along +Z axis.

uint8 CYLINDER_HEIGHT=0
uint8 CYLINDER_RADIUS=1

uint8 CONE_HEIGHT=0
uint8 CONE_RADIUS=1

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: shape_msgs/Plane
# Representation of a plane, using the plane equation ax + by + cz + d = 0

# a := coef[0]
# b := coef[1]
# c := coef[2]
# d := coef[3]

float64[4] coef

================================================================================
MSG: octomap_msgs/OctomapWithPose
# A 3D map in binary format, as Octree
Header header

# The pose of the octree with respect to the header frame 
geometry_msgs/Pose origin

# The actual octree msg
octomap_msgs/Octomap octomap

================================================================================
MSG: octomap_msgs/Octomap
# A 3D map in binary format, as Octree
Header header

# Flag to denote a binary (only free/occupied) or full occupancy octree (.bt/.ot file)
bool binary

# Class id of the contained octree 
string id

# Resolution (in m) of the smallest octree nodes
float64 resolution

# binary serialization of octree, use conversions.h to read and write octrees
int8[] data

"
  end
  attr_accessor :collision_objects, :octomap

  @@struct_C = ::ROS::Struct.new("C")
  @@struct_d = ::ROS::Struct.new("d")
  @@struct_d7L3 = ::ROS::Struct.new("d7L3")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_c = ::ROS::Struct.new("c")
  @@struct_d4 = ::ROS::Struct.new("d4")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['moveit_msgs/CollisionObject[]','octomap_msgs/OctomapWithPose']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [moveit_msgs/CollisionObject[]] :collision_objects initialize value
  # @option args [octomap_msgs/OctomapWithPose] :octomap initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:collision_objects]
      @collision_objects = args[:collision_objects]
    else
      @collision_objects = []
    end
    if args[:octomap]
      @octomap = args[:octomap]
    else
      @octomap = Octomap_msgs::OctomapWithPose.new
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      length = @collision_objects.length
      buff.write(@@struct_L.pack(length))
      for val1 in @collision_objects
        _v3385 = val1.header
        buff.write(@@struct_L.pack(_v3385.seq))
        _v3386 = _v3385.stamp
        _x = _v3386
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v3385.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1.id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _v3387 = val1.type
        _x = _v3387.key
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = _v3387.db
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        length = val1.primitives.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.primitives
          buff.write(@@struct_C.pack(val2.type))
          length = val2.dimensions.length
          buff.write(@@struct_L.pack(length))
          pattern = "d#{length}"
          buff.write(*val2.dimensions.pack(pattern))
        end
        length = val1.primitive_poses.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.primitive_poses
          _v3388 = val2.position
          _x = _v3388
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v3389 = val2.orientation
          _x = _v3389
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = val1.meshes.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.meshes
          length = val2.triangles.length
          buff.write(@@struct_L.pack(length))
          for val3 in val2.triangles
            buff.write(@@struct_L3.pack(*val3.vertex_indices))
          end
          length = val2.vertices.length
          buff.write(@@struct_L.pack(length))
          for val3 in val2.vertices
            _x = val3
            buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          end
        end
        length = val1.mesh_poses.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.mesh_poses
          _v3390 = val2.position
          _x = _v3390
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v3391 = val2.orientation
          _x = _v3391
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        length = val1.planes.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.planes
          buff.write(@@struct_d4.pack(*val2.coef))
        end
        length = val1.plane_poses.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.plane_poses
          _v3392 = val2.position
          _x = _v3392
          buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
          _v3393 = val2.orientation
          _x = _v3393
          buff.write(@@struct_d4.pack(_x.x, _x.y, _x.z, _x.w))
        end
        buff.write(@@struct_c.pack(val1.operation))
      end
      buff.write(@@struct_L3.pack(@octomap.header.seq, @octomap.header.stamp.secs, @octomap.header.stamp.nsecs))
      _x = @octomap.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d7L3.pack(@octomap.origin.position.x, @octomap.origin.position.y, @octomap.origin.position.z, @octomap.origin.orientation.x, @octomap.origin.orientation.y, @octomap.origin.orientation.z, @octomap.origin.orientation.w, @octomap.octomap.header.seq, @octomap.octomap.header.stamp.secs, @octomap.octomap.header.stamp.nsecs))
      _x = @octomap.octomap.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_C.pack(@octomap.octomap.binary))
      _x = @octomap.octomap.id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d.pack(@octomap.octomap.resolution))
      length = @octomap.octomap.data.length
      buff.write(@@struct_L.pack(length))
      pattern = "c#{length}"
      buff.write(*@octomap.octomap.data.pack(pattern))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @collision_objects == nil
        @collision_objects = Moveit_msgs::CollisionObject[].new
      end
      if @octomap == nil
        @octomap = Octomap_msgs::OctomapWithPose.new
      end
      end_point = 0
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @collision_objects = []
      length.times do
        val1 = Moveit_msgs::CollisionObject.new
        _v3394 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v3394.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v3395 = _v3394.stamp
        _x = _v3395
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v3394.frame_id = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.id = str[start..(end_point-1)]
        _v3396 = val1.type
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v3396.key = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v3396.db = str[start..(end_point-1)]
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.primitives = []
        length.times do
          val2 = Shape_msgs::SolidPrimitive.new
          start = end_point
          end_point += ROS::Struct::calc_size('C')
          (val2.type,) = @@struct_C.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          pattern = "d#{length}"
          start = end_point
          end_point += ROS::Struct::calc_size("#{pattern}")
          val2.dimensions = str[start..(end_point-1)].unpack(pattern)
          val1.primitives.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.primitive_poses = []
        length.times do
          val2 = Geometry_msgs::Pose.new
          _v3397 = val2.position
          _x = _v3397
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v3398 = val2.orientation
          _x = _v3398
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.primitive_poses.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.meshes = []
        length.times do
          val2 = Shape_msgs::Mesh.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val2.triangles = []
          length.times do
            val3 = Shape_msgs::MeshTriangle.new
            start = end_point
            end_point += ROS::Struct::calc_size('L3')
            val3.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
            val2.triangles.push(val3)
          end
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          val2.vertices = []
          length.times do
            val3 = Geometry_msgs::Point.new
            _x = val3
            start = end_point
            end_point += ROS::Struct::calc_size('d3')
            (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
            val2.vertices.push(val3)
          end
          val1.meshes.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.mesh_poses = []
        length.times do
          val2 = Geometry_msgs::Pose.new
          _v3399 = val2.position
          _x = _v3399
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v3400 = val2.orientation
          _x = _v3400
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.mesh_poses.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.planes = []
        length.times do
          val2 = Shape_msgs::Plane.new
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          val2.coef = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.planes.push(val2)
        end
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.plane_poses = []
        length.times do
          val2 = Geometry_msgs::Pose.new
          _v3401 = val2.position
          _x = _v3401
          start = end_point
          end_point += ROS::Struct::calc_size('d3')
          (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
          _v3402 = val2.orientation
          _x = _v3402
          start = end_point
          end_point += ROS::Struct::calc_size('d4')
          (_x.x, _x.y, _x.z, _x.w,) = @@struct_d4.unpack(str[start..(end_point-1)])
          val1.plane_poses.push(val2)
        end
        start = end_point
        end_point += ROS::Struct::calc_size('c')
        (val1.operation,) = @@struct_c.unpack(str[start..(end_point-1)])
        @collision_objects.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@octomap.header.seq, @octomap.header.stamp.secs, @octomap.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @octomap.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d7L3')
      (@octomap.origin.position.x, @octomap.origin.position.y, @octomap.origin.position.z, @octomap.origin.orientation.x, @octomap.origin.orientation.y, @octomap.origin.orientation.z, @octomap.origin.orientation.w, @octomap.octomap.header.seq, @octomap.octomap.header.stamp.secs, @octomap.octomap.header.stamp.nsecs,) = @@struct_d7L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @octomap.octomap.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('C')
      (@octomap.octomap.binary,) = @@struct_C.unpack(str[start..(end_point-1)])
      @octomap.octomap.binary = bool(@octomap.octomap.binary)
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @octomap.octomap.id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d')
      (@octomap.octomap.resolution,) = @@struct_d.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "c#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @octomap.octomap.data = str[start..(end_point-1)].unpack(pattern)
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
