# autogenerated by genrb from VisibilityConstraint.msg. Do not edit.
require 'ros/message'

require "std_msgs/Header"
require "geometry_msgs/Quaternion"
require "geometry_msgs/Point"
require "geometry_msgs/Pose"
require "geometry_msgs/PoseStamped"

module Moveit_msgs

class VisibilityConstraint <::ROS::Message
  def self.md5sum
    "62cda903bfe31ff2e5fcdc3810d577ad"
  end

  def self.type
    "moveit_msgs/VisibilityConstraint"
  end

  def has_header?
    false
  end

  def message_definition
    "# The constraint is useful to maintain visibility to a disc (the target) in a particular frame.
# This disc forms the base of a visibiliy cone whose tip is at the origin of the sensor.
# Maintaining visibility is done by ensuring the robot does not obstruct the visibility cone.
# Note:
# This constraint does NOT enforce minimum or maximum distances between the sensor
# and the target, nor does it enforce the target to be in the field of view of
# the sensor. A PositionConstraint can (and probably should) be used for such purposes.

# The radius of the disc that should be maintained visible 
float64 target_radius

# The pose of the disc; as the robot moves, the pose of the disc may change as well
# This can be in the frame of a particular robot link, for example
geometry_msgs/PoseStamped target_pose

# From the sensor origin towards the target, the disc forms a visibility cone
# This cone is approximated using many sides. For example, when using 4 sides, 
# that in fact makes the visibility region be a pyramid.
# This value should always be 3 or more.
int32 cone_sides

# The pose in which visibility is to be maintained.
# The frame id should represent the robot link to which the sensor is attached.
# It is assumed the sensor can look directly at the target, in any direction.
# This assumption is usually not true, but additional PositionConstraints
# can resolve this issue.
geometry_msgs/PoseStamped sensor_pose

# Even though the disc is maintained visible, the visibility cone can be very small
# because of the orientation of the disc with respect to the sensor. It is possible
# for example to view the disk almost from a side, in which case the visibility cone 
# can end up having close to 0 volume. The view angle is defined to be the angle between
# the normal to the visibility disc and the direction vector from the sensor origin.
# The value below represents the minimum desired view angle. For a perfect view,
# this value will be 0 (the two vectors are exact opposites). For a completely obstructed view
# this value will be Pi/2 (the vectors are perpendicular). This value defined below 
# is the maximum view angle to be maintained. This should be a value in the open interval
# (0, Pi/2). If 0 is set, the view angle is NOT enforced.
float64 max_view_angle

# This angle is used similarly to max_view_angle but limits the maximum angle
# between the sensor origin direction vector and the axis that connects the
# sensor origin to the target frame origin. The value is again in the range (0, Pi/2)
# and is NOT enforced if set to 0.
float64 max_range_angle

# The axis that is assumed to indicate the direction of view for the sensor
# X = 2, Y = 1, Z = 0
uint8 SENSOR_Z=0
uint8 SENSOR_Y=1
uint8 SENSOR_X=2
uint8 sensor_view_direction

# A weighting factor for this constraint (denotes relative importance to other constraints. Closer to zero means less important)
float64 weight

================================================================================
MSG: geometry_msgs/PoseStamped
# A Pose with reference coordinate frame and timestamp
Header header
Pose pose

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

"
  end
  # Pseudo-constants
  SENSOR_Z = 0
  SENSOR_Y = 1
  SENSOR_X = 2

  attr_accessor :target_radius, :target_pose, :cone_sides, :sensor_pose, :max_view_angle, :max_range_angle, :sensor_view_direction, :weight

  @@struct_d7lL3 = ::ROS::Struct.new("d7lL3")
  @@struct_d9Cd = ::ROS::Struct.new("d9Cd")
  @@struct_dL3 = ::ROS::Struct.new("dL3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['float64','geometry_msgs/PoseStamped','int32','geometry_msgs/PoseStamped','float64','float64','uint8','float64']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [float64] :target_radius initialize value
  # @option args [geometry_msgs/PoseStamped] :target_pose initialize value
  # @option args [int32] :cone_sides initialize value
  # @option args [geometry_msgs/PoseStamped] :sensor_pose initialize value
  # @option args [float64] :max_view_angle initialize value
  # @option args [float64] :max_range_angle initialize value
  # @option args [uint8] :sensor_view_direction initialize value
  # @option args [float64] :weight initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:target_radius]
      @target_radius = args[:target_radius]
    else
      @target_radius = 0.0
    end
    if args[:target_pose]
      @target_pose = args[:target_pose]
    else
      @target_pose = Geometry_msgs::PoseStamped.new
    end
    if args[:cone_sides]
      @cone_sides = args[:cone_sides]
    else
      @cone_sides = 0
    end
    if args[:sensor_pose]
      @sensor_pose = args[:sensor_pose]
    else
      @sensor_pose = Geometry_msgs::PoseStamped.new
    end
    if args[:max_view_angle]
      @max_view_angle = args[:max_view_angle]
    else
      @max_view_angle = 0.0
    end
    if args[:max_range_angle]
      @max_range_angle = args[:max_range_angle]
    else
      @max_range_angle = 0.0
    end
    if args[:sensor_view_direction]
      @sensor_view_direction = args[:sensor_view_direction]
    else
      @sensor_view_direction = 0
    end
    if args[:weight]
      @weight = args[:weight]
    else
      @weight = 0.0
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_dL3.pack(@target_radius, @target_pose.header.seq, @target_pose.header.stamp.secs, @target_pose.header.stamp.nsecs))
      _x = @target_pose.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d7lL3.pack(@target_pose.pose.position.x, @target_pose.pose.position.y, @target_pose.pose.position.z, @target_pose.pose.orientation.x, @target_pose.pose.orientation.y, @target_pose.pose.orientation.z, @target_pose.pose.orientation.w, @cone_sides, @sensor_pose.header.seq, @sensor_pose.header.stamp.secs, @sensor_pose.header.stamp.nsecs))
      _x = @sensor_pose.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d9Cd.pack(@sensor_pose.pose.position.x, @sensor_pose.pose.position.y, @sensor_pose.pose.position.z, @sensor_pose.pose.orientation.x, @sensor_pose.pose.orientation.y, @sensor_pose.pose.orientation.z, @sensor_pose.pose.orientation.w, @max_view_angle, @max_range_angle, @sensor_view_direction, @weight))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @target_pose == nil
        @target_pose = Geometry_msgs::PoseStamped.new
      end
      if @sensor_pose == nil
        @sensor_pose = Geometry_msgs::PoseStamped.new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('dL3')
      (@target_radius, @target_pose.header.seq, @target_pose.header.stamp.secs, @target_pose.header.stamp.nsecs,) = @@struct_dL3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @target_pose.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d7lL3')
      (@target_pose.pose.position.x, @target_pose.pose.position.y, @target_pose.pose.position.z, @target_pose.pose.orientation.x, @target_pose.pose.orientation.y, @target_pose.pose.orientation.z, @target_pose.pose.orientation.w, @cone_sides, @sensor_pose.header.seq, @sensor_pose.header.stamp.secs, @sensor_pose.header.stamp.nsecs,) = @@struct_d7lL3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @sensor_pose.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d9Cd')
      (@sensor_pose.pose.position.x, @sensor_pose.pose.position.y, @sensor_pose.pose.position.z, @sensor_pose.pose.orientation.x, @sensor_pose.pose.orientation.y, @sensor_pose.pose.orientation.z, @sensor_pose.pose.orientation.w, @max_view_angle, @max_range_angle, @sensor_view_direction, @weight,) = @@struct_d9Cd.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
