# autogenerated by genrb from RecognizedObject.msg. Do not edit.
require 'ros/message'

require "shape_msgs/MeshTriangle"
require "sensor_msgs/PointField"
require "geometry_msgs/PoseWithCovariance"
require "object_recognition_msgs/ObjectType"
require "std_msgs/Header"
require "geometry_msgs/Point"
require "shape_msgs/Mesh"
require "geometry_msgs/Pose"
require "geometry_msgs/Quaternion"
require "sensor_msgs/PointCloud2"
require "geometry_msgs/PoseWithCovarianceStamped"

module Object_recognition_msgs

class RecognizedObject <::ROS::Message
  def self.md5sum
    "f92c4cb29ba11f26c5f7219de97e900d"
  end

  def self.type
    "object_recognition_msgs/RecognizedObject"
  end

  def has_header?
    true
  end

  def message_definition
    "##################################################### HEADER ###########################################################

# The header frame corresponds to the pose frame, NOT the point_cloud frame.
Header header

################################################## OBJECT INFO #########################################################

# Contains information about the type and the position of a found object
# Some of those fields might not be filled because the used techniques do not fill them or because the user does not
# request them

# The type of the found object
object_recognition_msgs/ObjectType type

#confidence: how sure you are it is that object and not another one.
# It is between 0 and 1 and the closer to one it is the better
float32 confidence

################################################ OBJECT CLUSTERS #######################################################

# Sometimes you can extract the 3d points that belong to the object, in the frames of the original sensors
# (it is an array as you might have several sensors)
sensor_msgs/PointCloud2[] point_clouds

# Sometimes, you can only provide a bounding box/shape, even in 3d
# This is in the pose frame
shape_msgs/Mesh bounding_mesh

# Sometimes, you only have 2d input so you can't really give a pose, you just get a contour, or a box
# The last point will be linked to the first one automatically
geometry_msgs/Point[] bounding_contours

#################################################### POSE INFO #########################################################

# This is the result that everybody expects : the pose in some frame given with the input. The units are radian/meters
# as usual
geometry_msgs/PoseWithCovarianceStamped pose

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: object_recognition_msgs/ObjectType
################################################## OBJECT ID #########################################################

# Contains information about the type of a found object. Those two sets of parameters together uniquely define an
# object

# The key of the found object: the unique identifier in the given db
string key

# The db parameters stored as a JSON/compressed YAML string. An object id does not make sense without the corresponding
# database. E.g., in object_recognition, it can look like: \"{'type':'CouchDB', 'root':'http://localhost'}\"
# There is no conventional format for those parameters and it's nice to keep that flexibility.
# The object_recognition_core as a generic DB type that can read those fields
# Current examples:
# For CouchDB:
#   type: 'CouchDB'
#   root: 'http://localhost:5984'
#   collection: 'object_recognition'
# For SQL household database:
#   type: 'SqlHousehold'
#   host: 'wgs36'
#   port: 5432
#   user: 'willow'
#   password: 'willow'
#   name: 'household_objects'
#   module: 'tabletop'
string db

================================================================================
MSG: sensor_msgs/PointCloud2
# This message holds a collection of N-dimensional points, which may
# contain additional information such as normals, intensity, etc. The
# point data is stored as a binary blob, its layout described by the
# contents of the \"fields\" array.

# The point cloud data may be organized 2d (image-like) or 1d
# (unordered). Point clouds organized as 2d images may be produced by
# camera depth sensors such as stereo or time-of-flight.

# Time of sensor data acquisition, and the coordinate frame ID (for 3d
# points).
Header header

# 2D structure of the point cloud. If the cloud is unordered, height is
# 1 and width is the length of the point cloud.
uint32 height
uint32 width

# Describes the channels and their layout in the binary data blob.
PointField[] fields

bool    is_bigendian # Is this data bigendian?
uint32  point_step   # Length of a point in bytes
uint32  row_step     # Length of a row in bytes
uint8[] data         # Actual point data, size is (row_step*height)

bool is_dense        # True if there are no invalid points

================================================================================
MSG: sensor_msgs/PointField
# This message holds the description of one point entry in the
# PointCloud2 message format.
uint8 INT8    = 1
uint8 UINT8   = 2
uint8 INT16   = 3
uint8 UINT16  = 4
uint8 INT32   = 5
uint8 UINT32  = 6
uint8 FLOAT32 = 7
uint8 FLOAT64 = 8

string name      # Name of field
uint32 offset    # Offset from start of point struct
uint8  datatype  # Datatype enumeration, see above
uint32 count     # How many elements in the field

================================================================================
MSG: shape_msgs/Mesh
# Definition of a mesh

# list of triangles; the index values refer to positions in vertices[]
MeshTriangle[] triangles

# the actual vertices that make up the mesh
geometry_msgs/Point[] vertices

================================================================================
MSG: shape_msgs/MeshTriangle
# Definition of a triangle's vertices
uint32[3] vertex_indices

================================================================================
MSG: geometry_msgs/Point
# This contains the position of a point in free space
float64 x
float64 y
float64 z

================================================================================
MSG: geometry_msgs/PoseWithCovarianceStamped
# This expresses an estimated pose with a reference coordinate frame and timestamp

Header header
PoseWithCovariance pose

================================================================================
MSG: geometry_msgs/PoseWithCovariance
# This represents a pose in free space with uncertainty.

Pose pose

# Row-major representation of the 6x6 covariance matrix
# The orientation parameters use a fixed-axis representation.
# In order, the parameters are:
# (x, y, z, rotation about X axis, rotation about Y axis, rotation about Z axis)
float64[36] covariance

================================================================================
MSG: geometry_msgs/Pose
# A representation of pose in free space, composed of postion and orientation. 
Point position
Quaternion orientation

================================================================================
MSG: geometry_msgs/Quaternion
# This represents an orientation in free space in quaternion form.

float64 x
float64 y
float64 z
float64 w

"
  end
  attr_accessor :header, :type, :confidence, :point_clouds, :bounding_mesh, :bounding_contours, :pose

  @@struct_LCL = ::ROS::Struct.new("LCL")
  @@struct_C = ::ROS::Struct.new("C")
  @@struct_f = ::ROS::Struct.new("f")
  @@struct_d36 = ::ROS::Struct.new("d36")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d7 = ::ROS::Struct.new("d7")
  @@struct_CL2 = ::ROS::Struct.new("CL2")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_d3 = ::ROS::Struct.new("d3")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['std_msgs/Header','object_recognition_msgs/ObjectType','float32','sensor_msgs/PointCloud2[]','shape_msgs/Mesh','geometry_msgs/Point[]','geometry_msgs/PoseWithCovarianceStamped']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [std_msgs/Header] :header initialize value
  # @option args [object_recognition_msgs/ObjectType] :type initialize value
  # @option args [float32] :confidence initialize value
  # @option args [sensor_msgs/PointCloud2[]] :point_clouds initialize value
  # @option args [shape_msgs/Mesh] :bounding_mesh initialize value
  # @option args [geometry_msgs/Point[]] :bounding_contours initialize value
  # @option args [geometry_msgs/PoseWithCovarianceStamped] :pose initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:header]
      @header = args[:header]
    else
      @header = Std_msgs::Header.new
    end
    if args[:type]
      @type = args[:type]
    else
      @type = Object_recognition_msgs::ObjectType.new
    end
    if args[:confidence]
      @confidence = args[:confidence]
    else
      @confidence = 0.0
    end
    if args[:point_clouds]
      @point_clouds = args[:point_clouds]
    else
      @point_clouds = []
    end
    if args[:bounding_mesh]
      @bounding_mesh = args[:bounding_mesh]
    else
      @bounding_mesh = Shape_msgs::Mesh.new
    end
    if args[:bounding_contours]
      @bounding_contours = args[:bounding_contours]
    else
      @bounding_contours = []
    end
    if args[:pose]
      @pose = args[:pose]
    else
      @pose = Geometry_msgs::PoseWithCovarianceStamped.new
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_L3.pack(@header.seq, @header.stamp.secs, @header.stamp.nsecs))
      _x = @header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @type.key
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      _x = @type.db
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_f.pack(@confidence))
      length = @point_clouds.length
      buff.write(@@struct_L.pack(length))
      for val1 in @point_clouds
        _v107 = val1.header
        buff.write(@@struct_L.pack(_v107.seq))
        _v108 = _v107.stamp
        _x = _v108
        buff.write(@@struct_L2.pack(_x.secs, _x.nsecs))
        _x = _v107.frame_id
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        _x = val1
        buff.write(@@struct_L2.pack(_x.height, _x.width))
        length = val1.fields.length
        buff.write(@@struct_L.pack(length))
        for val2 in val1.fields
          _x = val2.name
          length = _x.length
          buff.write([length, _x].pack("La#{length}"))
          _x = val2
          buff.write(@@struct_LCL.pack(_x.offset, _x.datatype, _x.count))
        end
        _x = val1
        buff.write(@@struct_CL2.pack(_x.is_bigendian, _x.point_step, _x.row_step))
        _x = val1.data
        length = _x.length
        buff.write([length, _x].pack("La#{length}"))
        buff.write(@@struct_C.pack(val1.is_dense))
      end
      length = @bounding_mesh.triangles.length
      buff.write(@@struct_L.pack(length))
      for val1 in @bounding_mesh.triangles
        buff.write(@@struct_L3.pack(*val1.vertex_indices))
      end
      length = @bounding_mesh.vertices.length
      buff.write(@@struct_L.pack(length))
      for val1 in @bounding_mesh.vertices
        _x = val1
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
      length = @bounding_contours.length
      buff.write(@@struct_L.pack(length))
      for val1 in @bounding_contours
        _x = val1
        buff.write(@@struct_d3.pack(_x.x, _x.y, _x.z))
      end
      buff.write(@@struct_L3.pack(@pose.header.seq, @pose.header.stamp.secs, @pose.header.stamp.nsecs))
      _x = @pose.header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_d7.pack(@pose.pose.pose.position.x, @pose.pose.pose.position.y, @pose.pose.pose.position.z, @pose.pose.pose.orientation.x, @pose.pose.pose.orientation.y, @pose.pose.pose.orientation.z, @pose.pose.pose.orientation.w))
      buff.write(@@struct_d36.pack(*@pose.pose.covariance))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @header == nil
        @header = Std_msgs::Header.new
      end
      if @type == nil
        @type = Object_recognition_msgs::ObjectType.new
      end
      if @point_clouds == nil
        @point_clouds = Sensor_msgs::PointCloud2[].new
      end
      if @bounding_mesh == nil
        @bounding_mesh = Shape_msgs::Mesh.new
      end
      if @bounding_contours == nil
        @bounding_contours = Geometry_msgs::Point[].new
      end
      if @pose == nil
        @pose = Geometry_msgs::PoseWithCovarianceStamped.new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@header.seq, @header.stamp.secs, @header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.key = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @type.db = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('f')
      (@confidence,) = @@struct_f.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @point_clouds = []
      length.times do
        val1 = Sensor_msgs::PointCloud2.new
        _v109 = val1.header
        start = end_point
        end_point += ROS::Struct::calc_size('L')
        (_v109.seq,) = @@struct_L.unpack(str[start..(end_point-1)])
        _v110 = _v109.stamp
        _x = _v110
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.secs, _x.nsecs,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        _v109.frame_id = str[start..(end_point-1)]
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('L2')
        (_x.height, _x.width,) = @@struct_L2.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        val1.fields = []
        length.times do
          val2 = Sensor_msgs::PointField.new
          start = end_point
          end_point += 4
          (length,) = @@struct_L.unpack(str[start..(end_point-1)])
          start = end_point
          end_point += length
          val2.name = str[start..(end_point-1)]
          _x = val2
          start = end_point
          end_point += ROS::Struct::calc_size('LCL')
          (_x.offset, _x.datatype, _x.count,) = @@struct_LCL.unpack(str[start..(end_point-1)])
          val1.fields.push(val2)
        end
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('CL2')
        (_x.is_bigendian, _x.point_step, _x.row_step,) = @@struct_CL2.unpack(str[start..(end_point-1)])
        val1.is_bigendian = bool(val1.is_bigendian)
        start = end_point
        end_point += 4
        (length,) = @@struct_L.unpack(str[start..(end_point-1)])
        start = end_point
        end_point += length
        val1.data = str[start..(end_point-1)]
        start = end_point
        end_point += ROS::Struct::calc_size('C')
        (val1.is_dense,) = @@struct_C.unpack(str[start..(end_point-1)])
        val1.is_dense = bool(val1.is_dense)
        @point_clouds.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @bounding_mesh.triangles = []
      length.times do
        val1 = Shape_msgs::MeshTriangle.new
        start = end_point
        end_point += ROS::Struct::calc_size('L3')
        val1.vertex_indices = @@struct_L3.unpack(str[start..(end_point-1)])
        @bounding_mesh.triangles.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @bounding_mesh.vertices = []
      length.times do
        val1 = Geometry_msgs::Point.new
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @bounding_mesh.vertices.push(val1)
      end
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      @bounding_contours = []
      length.times do
        val1 = Geometry_msgs::Point.new
        _x = val1
        start = end_point
        end_point += ROS::Struct::calc_size('d3')
        (_x.x, _x.y, _x.z,) = @@struct_d3.unpack(str[start..(end_point-1)])
        @bounding_contours.push(val1)
      end
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@pose.header.seq, @pose.header.stamp.secs, @pose.header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @pose.header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('d7')
      (@pose.pose.pose.position.x, @pose.pose.pose.position.y, @pose.pose.pose.position.z, @pose.pose.pose.orientation.x, @pose.pose.pose.orientation.y, @pose.pose.pose.orientation.z, @pose.pose.pose.orientation.w,) = @@struct_d7.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('d36')
      @pose.pose.covariance = @@struct_d36.unpack(str[start..(end_point-1)])
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
