# autogenerated by genrb from CameraInfo.msg. Do not edit.
require 'ros/message'

require "std_msgs/Header"
require "sensor_msgs/RegionOfInterest"

module Sensor_msgs

class CameraInfo <::ROS::Message
  def self.md5sum
    "c9a58c1b0b154e0e6da7578cb991d214"
  end

  def self.type
    "sensor_msgs/CameraInfo"
  end

  def has_header?
    true
  end

  def message_definition
    "# This message defines meta information for a camera. It should be in a
# camera namespace on topic \"camera_info\" and accompanied by up to five
# image topics named:
#
#   image_raw - raw data from the camera driver, possibly Bayer encoded
#   image            - monochrome, distorted
#   image_color      - color, distorted
#   image_rect       - monochrome, rectified
#   image_rect_color - color, rectified
#
# The image_pipeline contains packages (image_proc, stereo_image_proc)
# for producing the four processed image topics from image_raw and
# camera_info. The meaning of the camera parameters are described in
# detail at http://www.ros.org/wiki/image_pipeline/CameraInfo.
#
# The image_geometry package provides a user-friendly interface to
# common operations using this meta information. If you want to, e.g.,
# project a 3d point into image coordinates, we strongly recommend
# using image_geometry.
#
# If the camera is uncalibrated, the matrices D, K, R, P should be left
# zeroed out. In particular, clients may assume that K[0] == 0.0
# indicates an uncalibrated camera.

#######################################################################
#                     Image acquisition info                          #
#######################################################################

# Time of image acquisition, camera coordinate frame ID
Header header    # Header timestamp should be acquisition time of image
                 # Header frame_id should be optical frame of camera
                 # origin of frame should be optical center of camera
                 # +x should point to the right in the image
                 # +y should point down in the image
                 # +z should point into the plane of the image


#######################################################################
#                      Calibration Parameters                         #
#######################################################################
# These are fixed during camera calibration. Their values will be the #
# same in all messages until the camera is recalibrated. Note that    #
# self-calibrating systems may \"recalibrate\" frequently.              #
#                                                                     #
# The internal parameters can be used to warp a raw (distorted) image #
# to:                                                                 #
#   1. An undistorted image (requires D and K)                        #
#   2. A rectified image (requires D, K, R)                           #
# The projection matrix P projects 3D points into the rectified image.#
#######################################################################

# The image dimensions with which the camera was calibrated. Normally
# this will be the full camera resolution in pixels.
uint32 height
uint32 width

# The distortion model used. Supported models are listed in
# sensor_msgs/distortion_models.h. For most cameras, \"plumb_bob\" - a
# simple model of radial and tangential distortion - is sufficent.
string distortion_model

# The distortion parameters, size depending on the distortion model.
# For \"plumb_bob\", the 5 parameters are: (k1, k2, t1, t2, k3).
float64[] D

# Intrinsic camera matrix for the raw (distorted) images.
#     [fx  0 cx]
# K = [ 0 fy cy]
#     [ 0  0  1]
# Projects 3D points in the camera coordinate frame to 2D pixel
# coordinates using the focal lengths (fx, fy) and principal point
# (cx, cy).
float64[9]  K # 3x3 row-major matrix

# Rectification matrix (stereo cameras only)
# A rotation matrix aligning the camera coordinate system to the ideal
# stereo image plane so that epipolar lines in both stereo images are
# parallel.
float64[9]  R # 3x3 row-major matrix

# Projection/camera matrix
#     [fx'  0  cx' Tx]
# P = [ 0  fy' cy' Ty]
#     [ 0   0   1   0]
# By convention, this matrix specifies the intrinsic (camera) matrix
#  of the processed (rectified) image. That is, the left 3x3 portion
#  is the normal camera intrinsic matrix for the rectified image.
# It projects 3D points in the camera coordinate frame to 2D pixel
#  coordinates using the focal lengths (fx', fy') and principal point
#  (cx', cy') - these may differ from the values in K.
# For monocular cameras, Tx = Ty = 0. Normally, monocular cameras will
#  also have R = the identity and P[1:3,1:3] = K.
# For a stereo pair, the fourth column [Tx Ty 0]' is related to the
#  position of the optical center of the second camera in the first
#  camera's frame. We assume Tz = 0 so both cameras are in the same
#  stereo image plane. The first camera always has Tx = Ty = 0. For
#  the right (second) camera of a horizontal stereo pair, Ty = 0 and
#  Tx = -fx' * B, where B is the baseline between the cameras.
# Given a 3D point [X Y Z]', the projection (x, y) of the point onto
#  the rectified image is given by:
#  [u v w]' = P * [X Y Z 1]'
#         x = u / w
#         y = v / w
#  This holds for both images of a stereo pair.
float64[12] P # 3x4 row-major matrix


#######################################################################
#                      Operational Parameters                         #
#######################################################################
# These define the image region actually captured by the camera       #
# driver. Although they affect the geometry of the output image, they #
# may be changed freely without recalibrating the camera.             #
#######################################################################

# Binning refers here to any camera setting which combines rectangular
#  neighborhoods of pixels into larger \"super-pixels.\" It reduces the
#  resolution of the output image to
#  (width / binning_x) x (height / binning_y).
# The default values binning_x = binning_y = 0 is considered the same
#  as binning_x = binning_y = 1 (no subsampling).
uint32 binning_x
uint32 binning_y

# Region of interest (subwindow of full camera resolution), given in
#  full resolution (unbinned) image coordinates. A particular ROI
#  always denotes the same window of pixels on the camera sensor,
#  regardless of binning settings.
# The default setting of roi (all values 0) is considered the same as
#  full resolution (roi.width = width, roi.height = height).
RegionOfInterest roi

================================================================================
MSG: std_msgs/Header
# Standard metadata for higher-level stamped data types.
# This is generally used to communicate timestamped data 
# in a particular coordinate frame.
# 
# sequence ID: consecutively increasing ID 
uint32 seq
#Two-integer timestamp that is expressed as:
# * stamp.secs: seconds (stamp_secs) since epoch
# * stamp.nsecs: nanoseconds since stamp_secs
# time-handling sugar is provided by the client library
time stamp
#Frame this data is associated with
# 0: no frame
# 1: global frame
string frame_id

================================================================================
MSG: sensor_msgs/RegionOfInterest
# This message is used to specify a region of interest within an image.
#
# When used to specify the ROI setting of the camera when the image was
# taken, the height and width fields should either match the height and
# width fields for the associated image; or height = width = 0
# indicates that the full resolution image was captured.

uint32 x_offset  # Leftmost pixel of the ROI
                 # (0 if the ROI includes the left edge of the image)
uint32 y_offset  # Topmost pixel of the ROI
                 # (0 if the ROI includes the top edge of the image)
uint32 height    # Height of ROI
uint32 width     # Width of ROI

# True if a distinct rectified ROI should be calculated from the \"raw\"
# ROI in this message. Typically this should be False if the full image
# is captured (ROI not used), and True if a subwindow is captured (ROI
# used).
bool do_rectify

"
  end
  attr_accessor :header, :height, :width, :distortion_model, :D, :K, :R, :P, :binning_x, :binning_y, :roi

  @@struct_L6C = ::ROS::Struct.new("L6C")
  @@struct_L3 = ::ROS::Struct.new("L3")
  @@struct_d12 = ::ROS::Struct.new("d12")
  @@struct_L2 = ::ROS::Struct.new("L2")
  @@struct_d9 = ::ROS::Struct.new("d9")

  @@struct_L = ::ROS::Struct.new("L")
  @@slot_types = ['std_msgs/Header','uint32','uint32','string','float64[]','float64[9]','float64[9]','float64[12]','uint32','uint32','sensor_msgs/RegionOfInterest']

  # Constructor. You can set the default values using keyword operators.
  #
  # @param [Hash] args keyword for initializing values
  # @option args [std_msgs/Header] :header initialize value
  # @option args [uint32] :height initialize value
  # @option args [uint32] :width initialize value
  # @option args [string] :distortion_model initialize value
  # @option args [float64[]] :D initialize value
  # @option args [float64[9]] :K initialize value
  # @option args [float64[9]] :R initialize value
  # @option args [float64[12]] :P initialize value
  # @option args [uint32] :binning_x initialize value
  # @option args [uint32] :binning_y initialize value
  # @option args [sensor_msgs/RegionOfInterest] :roi initialize value
  def initialize(args={})
    # message fields cannot be None, assign default values for those that are
    if args[:header]
      @header = args[:header]
    else
      @header = Std_msgs::Header.new
    end
    if args[:height]
      @height = args[:height]
    else
      @height = 0
    end
    if args[:width]
      @width = args[:width]
    else
      @width = 0
    end
    if args[:distortion_model]
      @distortion_model = args[:distortion_model]
    else
      @distortion_model = ''
    end
    if args[:D]
      @D = args[:D]
    else
      @D = []
    end
    if args[:K]
      @K = args[:K]
    else
      @K = [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0]
    end
    if args[:R]
      @R = args[:R]
    else
      @R = [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0]
    end
    if args[:P]
      @P = args[:P]
    else
      @P = [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0]
    end
    if args[:binning_x]
      @binning_x = args[:binning_x]
    else
      @binning_x = 0
    end
    if args[:binning_y]
      @binning_y = args[:binning_y]
    else
      @binning_y = 0
    end
    if args[:roi]
      @roi = args[:roi]
    else
      @roi = Sensor_msgs::RegionOfInterest.new
    end
  end

  # internal API method
  # @return [String] Message type string.
  def _get_types
    @slot_types
  end

  # serialize message into buffer
  # @param [IO] buff buffer
  def serialize(buff)
    begin
      buff.write(@@struct_L3.pack(@header.seq, @header.stamp.secs, @header.stamp.nsecs))
      _x = @header.frame_id
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      buff.write(@@struct_L2.pack(@height, @width))
      _x = @distortion_model
      length = _x.length
      buff.write([length, _x].pack("La#{length}"))
      length = @D.length
      buff.write(@@struct_L.pack(length))
      pattern = "d#{length}"
      buff.write(*@D.pack(pattern))
      buff.write(@@struct_d9.pack(*@K))
      buff.write(@@struct_d9.pack(*@R))
      buff.write(@@struct_d12.pack(*@P))
      buff.write(@@struct_L6C.pack(@binning_x, @binning_y, @roi.x_offset, @roi.y_offset, @roi.height, @roi.width, @roi.do_rectify))
    rescue => exception
      raise "some erro in serialize: #{exception}"

    end
  end

  #  unpack serialized message in str into this message instance
  #  @param [String] str: byte array of serialized message
  def deserialize(str)

    begin
      if @header == nil
        @header = Std_msgs::Header.new
      end
      if @roi == nil
        @roi = Sensor_msgs::RegionOfInterest.new
      end
      end_point = 0
      start = end_point
      end_point += ROS::Struct::calc_size('L3')
      (@header.seq, @header.stamp.secs, @header.stamp.nsecs,) = @@struct_L3.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @header.frame_id = str[start..(end_point-1)]
      start = end_point
      end_point += ROS::Struct::calc_size('L2')
      (@height, @width,) = @@struct_L2.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += length
      @distortion_model = str[start..(end_point-1)]
      start = end_point
      end_point += 4
      (length,) = @@struct_L.unpack(str[start..(end_point-1)])
      pattern = "d#{length}"
      start = end_point
      end_point += ROS::Struct::calc_size("#{pattern}")
      @D = str[start..(end_point-1)].unpack(pattern)
      start = end_point
      end_point += ROS::Struct::calc_size('d9')
      @K = @@struct_d9.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('d9')
      @R = @@struct_d9.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('d12')
      @P = @@struct_d12.unpack(str[start..(end_point-1)])
      start = end_point
      end_point += ROS::Struct::calc_size('L6C')
      (@binning_x, @binning_y, @roi.x_offset, @roi.y_offset, @roi.height, @roi.width, @roi.do_rectify,) = @@struct_L6C.unpack(str[start..(end_point-1)])
      @roi.do_rectify = bool(@roi.do_rectify)
      return self
    rescue => exception
      raise "message DeserializationError: #{exception}" #most likely buffer underfill
    end
  end
end # end of class
end # end of module
